#!/usr/bin/env python3
import sys,time,glob,os,pickle,fileinput,argparse
from subprocess import Popen
from operator import itemgetter
import gc, fileinput
import numpy as np
from PIL import Image

from mana_clust.common_functions import *

########################################################

########################################################
########################################################

########################################################
############# add the various elements #################

def get_img_dims(img):
    image = Image.open(img)
    w, h = image.size
    ## reduce size so it fits on the screne
    reduction_factor = w/600
    return(int(w/reduction_factor), int(h/reduction_factor))

def add_h1(title):
    return("\n\t<h1>"+str(title)+"</h1>\n")

def add_h2(title):
    return("\n\t<h2>"+str(title)+"</h2>\n")

def add_h3(title):
    return("\n\t<h3>"+str(title)+"</h3>\n")

def add_p(p):
    return('\t\t\t<p>'+str(p)+'</p>\n')

def add_file_link(f,name):
    if not os.path.isfile(f):
        print("\n\tWARNING: couldn't find file:",f,"\n")
    return('<a href='+str(f)+'>'+str(name)+'</a>\n')

def add_file_link_list(f,name):
    return('<li><a href='+str(f)+'>'+str(name)+'</a></li>\n')

def add_br():
    return('<br></br>\n')

def add_img(img,alt, base_dir= None):
    if os.path.isfile(img):
        w, h = get_img_dims(img)
        return('<div><img src="'+str(img)+'" alt="'+str(alt)+'" width="'+str(w)+'" height="'+str(h)+'"></div>\n')
    else:
        if base_dir != None:
            curr_dir = os.getcwd()
            os.chdir(base_dir)
            final_text = add_img(img, alt)
            os.chdir(curr_dir)
            return(final_text)
        print("\tWARNING: couldn't find:"+img)
        return(add_br())

def add_button_head(text):
    return("""<button class="collapsible">"""+str(text)+"""</button>
<div class="content">""")

def add_table(in_table):
    ## start the table
    out_table_str = """<table style="width:100%">\n"""
    for i in range(len(in_table)):
        out_table_str+="\t<tr>\n"
        for j in range(len(in_table[i])):
            out_table_str+="\t<th>"+str(in_table[i][j])+"</th>\n"
        out_table_str+="\t</tr>\n"
    out_table_str+="</table>\n"
    return(out_table_str)


def parse_file_name(img):
    old_name = img[:-4]
    temp_name = old_name.split('_')
    temp_name = ' '.join(temp_name)
    return(old_name,temp_name)



def add_clustering(base_dir):
    os.chdir(base_dir)
    cluster_str = ""

    #cluster_str+=add_h2('Clustering')
    cluster_str+=add_button_head('Clustering')
    ## check if anti_correlation clustering was done
    if os.path.isfile(base_dir+'sig_neg_count_vs_total_neg_count.png'):
        ## if it is, start annotating
        cluster_str+=add_h3("Negative Control Bootstrap Shuffling")
        cluster_str+=add_p("Here, MANAclust shuffled up your data to make it randomized. This will maintain the overall distribution of your data, while at the same time randomizing it so that we can come up with a reasonable cutoff for performing the anti-correlation based feature selection. Shown below is the distribution of all randomized Spearman rhos.")
        cluster_str+=add_img("boostrap_cor_rhos.png", 'boot_all')
        cluster_str+=add_p("Here are just the negative correlations from the shuffled up dataset.")
        cluster_str+=add_img("boostrap_neg_cor_rhos.png", 'boot_neg')
        cluster_str+=add_p("Here is a scatter plot that shows the number of total negative correlations observed for each gene (y-axis), and the log2 number of significant negative correlations (x-axis). All of the genes to the left/below the black line were used for clustering.")
        cluster_str+=add_img("sig_neg_count_vs_total_neg_count.png","sig_vs_total_neg")
        cluster_str+=add_p("Similarly, here is a plot showing the ratio of significant to non-sigificant. Everything to the right was used for clustering.")

    ## link to the file with all of the sample annotations
    cluster_str+=add_h3("Clustering Results:")
    cluster_str+=add_file_link("sample_clustering_and_summary/sample_k_means_groups.txt",'sample group annotations')

    ## get the images from the sample_clustering_and_summary folder
    os.chdir(base_dir+"sample_clustering_and_summary/")
    additional_images=[]
    for item in glob.glob("*.png"):
        additional_images.append(item)
    os.chdir(base_dir)
    if len(additional_images)>0:
        cluster_str+=add_h3('Here are some additional images:')
        for img in additional_images:
            old_name = img[:-4]
            temp_name = old_name.split('_')
            temp_name = ' '.join(temp_name)
            cluster_str+=add_p(temp_name)
            cluster_str+=add_img("sample_clustering_and_summary/"+img,old_name)
    
    static_files = []
    cluster_str+='\n</div>\n'
    os.chdir(base_dir)
    return(cluster_str)



def add_statistics(base_dir):
    stats_str = ""
    #stats_str+=add_h2("Statistics")
    stats_str+=add_button_head("Statistics")
    stats_str+=add_h3("basic stats")
    stats_str+=add_p("If you find some interesting genes that are different between groups, here are the:")
    stats_str+=add_file_link("sample_clustering_and_summary/k_group_means.txt",'group means')
    stats_str+=add_p('and...')
    stats_str+=add_file_link("sample_clustering_and_summary/k_group_sd.txt",'group standard deviations')
    stats_str+=add_p('as well as the')
    # stats_str+=add_file_link("sample_clustering_and_summary/sample_var_enrichment_Zscores.txt",'sample-wise Z-scores')
    # stats_str+=add_p('and...')
    stats_str+=add_file_link("sample_clustering_and_summary/k_group_enrichment.txt",'group level Z-scores')
    stats_str+=add_h3("statistical comparisons")
    sig_dir = "sample_clustering_and_summary/significance/"

    stats_str+=add_file_link(sig_dir+'groups_1way_anova_results.txt',"Benjamini-Hoschberg corrected 1-way Anovas")
    stats_str+='\n</div>\n'
    return(stats_str)

def add_gene_enrichment(base_dir):
    os.chdir(base_dir)
    sig_dir = "sample_clustering_and_summary/significance/"
    enrich_str = ""
    #enrich_str+=add_h2("Gene Enrichment in Groups")
    enrich_str+=add_button_head("Gene enrichment in groups")
    enrich_str+=add_h3("Significantly enriched genes in each group")
    enrich_str+=add_p("Below is a file that contains a table with genes on the left, and groups in columns. If a gene is considered significantly enriched, that means that the BH corrected 1-way Anova was significant, and the gene had a high Z-score in that particular group. If a gene is significantly enriched in that group, the value is True in the corresponding cell in the table, False if it was not significantly enriched.")
    enrich_str+=add_file_link(sig_dir+"/significant_and_enriched_boolean_table.txt","True/False significantly enriched table")
    enrich_str+=add_p("alternatively, you can use these separated files that simply have the list of significantly enriched genes for each group:")
    enrich_str+=add_button_head('Genes enriched in each group')
    os.chdir(sig_dir)
    enrich_files=[]
    for item in glob.glob("*_significant_enriched.txt"):
        enrich_files.append(item)
    enrich_files = sorted(enrich_files)
    if len(enrich_files)>0:
        enrich_str+="\t\t\t\t<ul>\n"
        for img in enrich_files:
            old_name = img[:-4]
            temp_name = old_name.split('_')
            temp_name = ' '.join(temp_name)
            #enrich_str+=add_p(temp_name)
            enrich_str+=add_file_link(sig_dir+img,old_name)
            enrich_str+=add_br()

        enrich_str+="\t\t\t\t</ul>\n"

    enrich_str+="\t\t</div>"# end the sub-button
    enrich_str+=add_br()

    os.chdir(base_dir)
    pathway_dir = sig_dir+'gprofiler/'
    os.chdir(pathway_dir)
    enrich_path=[]
    for item in glob.glob("*.txt"):
        enrich_path.append(item)
    os.chdir(base_dir)
    enrich_path = sorted(enrich_path)
    if len(enrich_path)>0:
        #enrich_str+=add_h3('Pathway Enrichment for Each Group:')
        enrich_str+=add_button_head('Pathway enrichment for each group')
        if os.path.isfile(sig_dir+"/combined_neg_log10p_gprofiler.txt"):
            enrich_str+=add_p("Below is a combined file with all of the pathways that came out of the analysis of the above genes, enriched in the different groups. We've also created a new algorithm for ranking the importance of these pathways (not to boast, but I'm kind of proud of it). It's based on the information/entropy of the -log10(p-vals). If you look at the -log10(p-vals) at the top of the list, you should find pathways that are really high in some groups and really low in other groups. The overall formula is calculated by taking the sum(KL-divergance)*range(-log10(p-vals)). The KL-divergence is looking for a difference in the distribution between the observed -log10(p-vals) and the distribution expected from an uninformative vector of -log10(p-vals) (this is a Gaussian null hypothesis). Then we multiply the sum(KL-divergance) by the range of -log10(p-vals) to let the most significant pathways with lots of information/low entropy rise to the top.")
            enrich_str+=add_file_link(sig_dir+"/combined_neg_log10p_gprofiler.txt",'combined pathway analyses')
            enrich_str+=add_p("and here is a file that has a normalized metric that ranks each pathway for their individual importance within each group. It would be useful to sort each of these and see what rises to the top for each 'cell type' or whatever your groups are.")
            enrich_str+=add_file_link(sig_dir+'/individual_class_importance.txt', 'individual class importance')
            enrich_str+=add_p("Below are all of the individual results so that you find which genes were in which pathways in individual groups:")
            enrich_str+=add_button_head("individual pathway files")
        enrich_str+="\t\t\t\t<ul>\n"
        for img in enrich_path:
            old_name = img[:-4]
            temp_name = old_name.split('_')
            temp_name = ' '.join(temp_name)
            #enrich_str+=add_p(temp_name)
            enrich_str+=add_file_link(pathway_dir+img,old_name)
            enrich_str+=add_br()


        if os.path.isfile(sig_dir+"/combined_neg_log10p_gprofiler.txt"):
            enrich_str+="\n\t\t</div>\n"

        enrich_str+="\t\t\t\t</ul>\n"
        enrich_str+="\n\t\t</div>\n"#end the sub-button
        enrich_str+=add_br()

    enrich_str+='\n</div>\n'
    return(enrich_str)


def add_community_pathway_annotation(com, individual_class_importance_table):
    if type(individual_class_importance_table) != np.ndarray:
        return(add_br())
    print(individual_class_importance_table)
    ## get the top 5 pathways
    ## first get the right column
    header = individual_class_importance_table[0,:]
    temp_col = np.where(header == com+"_gprofiler.txt")[0]
    print(temp_col)
    print(com,individual_class_importance_table[1:,temp_col])
    most_sig_order = np.argsort(np.array(individual_class_importance_table[1:,temp_col].tolist(),dtype=np.float32), axis = 0)[::-1] + 1
    #print(individual_class_importance_table[most_sig_order,temp_col])
    #print(most_sig_order)
    keep_rows = []
    for i in range(0,5):
        temp_row = most_sig_order[i]
        ## check if it's actually significant first
        print(individual_class_importance_table[temp_row,temp_col])
        if float(individual_class_importance_table[temp_row,temp_col]) > 0:
            keep_rows.append(temp_row)
    ## go through the rows that we're keeping & summarize them
    out_table = [["term_id","term_def","importance_metric"]]
    for i in keep_rows:
        temp_line = [individual_class_importance_table[i,1][0],individual_class_importance_table[i,3][0],individual_class_importance_table[i,temp_col][0]]
        out_table.append(temp_line)
    return(add_table(out_table)+add_br())


def add_community_top_markers(base_dir, com, community_annotations_file, top_n_markers = 15):
    com_marker_str = ""
    com_marker_str += add_button_head("top marker genes of "+com+" (defined by high Local PageRank)")
    ## read in the community annotations file
    try:
        com_anno_table = read_table(community_annotations_file)
    except:
        cur_dir = os.getcwd()
        os.chdir(base_dir)
        com_anno_table = read_table(community_annotations_file)
        os.chdir(cur_dir)
    else:
        com_anno_table = read_table(community_annotations_file)
    num_lines = min([len(com_anno_table),top_n_markers])
    temp_table = com_anno_table[:num_lines]
    # for i in range(len(temp_table)):
    #     print(temp_table[i])

    com_marker_str += add_p("this is a list of the top genes that are most central to this hub, and will therefore likely make good marker genes.")
    com_marker_str += add_table(temp_table)
    # print(add_table(temp_table))
    # sys.exit()
    com_marker_str += "</div>"## ends the button tab


    return(com_marker_str)



def add_community_str(base_dir, com, individual_class_importance_table):
    com_str = ""
    com_str += add_h3("Community module usage")
    com_str += add_community_pathway_annotation(com, individual_class_importance_table)
    com_dir = os.path.join(os.path.join("pos_cor_graphs/community_analysis/",com))
    community_annotations_file = os.path.join(com_dir,"community_ids_annotated.tsv")
    com_str += add_community_top_markers(base_dir, com, community_annotations_file)
    com_str += add_file_link(community_annotations_file,"This is the list of genes in the community.")
    com_str += add_br()
    com_str += add_file_link(os.path.join(com_dir,"TukeyHSD.tsv"),"This is the differential module usage across groups")
    com_str += add_p("And here is a plot showing the module usage across groups.")
    com_str += add_img(os.path.join(com_dir,"group_z_scores.png"),"Group Z-scores", base_dir = base_dir)
    com_str += add_br()
    com_str += add_p("This is where this community is in the network graph.")
    com_str += add_img(os.path.join(com_dir,"community.png"),"community subset", base_dir = base_dir)
    return(com_str)



def add_graph(base_dir):
    graph_str=""
    #graph_str+=add_h2("Expression graphs")
    graph_str+=add_button_head("Expression graphs")
    graph_str+=add_h3("Adjacency Lists:")
    adj_list_list = []
    os.chdir(base_dir)
    for f in glob.glob("*"):
        if 'adj_list' in f:
            adj_list_list.append(f)
    print(adj_list_list)
    for f in adj_list_list:
        if '_pos.tsv' in f:
            pos_adj = f
        elif '_neg.tsv' in f:
            neg_adj = f
        else:
            total_adj = f
    # graph_str+=add_p('Full adjacency list:')[:-6]+'</p>'
    # print(base_dir+total_adj)
    graph_str+="\t\t\t\t<ul>\n"
    graph_str+=add_file_link(total_adj,'Full adjacency list')
    graph_str+=add_br()
    graph_str+=add_file_link(pos_adj,'Positive correlation (co-expression) adjacency list')
    graph_str+=add_br()
    graph_str+=add_file_link(neg_adj,'Negative correlation adjacency list')
    graph_str+=add_br()
    graph_str+="\t\t\t\t</ul>\n"

    graph_str+=add_h2("Co-expression graph plots:")
    coexpression_graph_dir = "pos_cor_graphs/"
    graph_str+=add_h3("This is a 'hairball' view of a Spearman coexpression network of your data, where each point represents a gene, and the lines are whether or not a gene is correlated with another gene. The closer two genes are to each other, the more correlated they are.")
    graph_str+=add_img(coexpression_graph_dir+"full_graph.png",'Co-expression graph')
    if os.path.isfile(coexpression_graph_dir+"community.png"):
        graph_str+=add_h3("This is a graph of all the communities of genes that are coordinately regulated in your dataset")
        graph_str+=add_img(coexpression_graph_dir+"community.png",'communities')

    ## 
    community_dir = os.path.join(coexpression_graph_dir,"community_analysis")
    if os.path.isdir(community_dir):
        graph_str+=add_button_head('Analysis of individual communities')
        os.chdir(community_dir)
        ## first add the significance and pathway files
        graph_str+=add_br()
        graph_str+=add_file_link(os.path.join(base_dir, community_dir, "global_statistics.tsv"),"Here are the stats (BH corrected 1-way anova) for differential module usage among groups.")
        graph_str+=add_br()
        graph_str+=add_file_link(os.path.join(base_dir, community_dir, "combined_neg_log10p_gprofiler.txt"),"-log10(p-val) for the pathways associated with each module")
        graph_str+=add_br()
        individual_class_importance_file = os.path.join(base_dir, community_dir, "individual_class_importance.txt")
        graph_str+=add_file_link(individual_class_importance_file,"The unique pathways of each module (individual class importance).")
        graph_str+=add_br()
        ## second add the folders for each individual community
        print(os.getcwd())
        community_dirs = []
        for file in glob.glob('*'):
            if os.path.isdir(file):
                if file != 'gprofiler':
                    print(file)
                    community_dirs.append(file)

        ## catelogue how many genes are in each community
        com_num_dict = {}
        com_num_list = []
        for com in community_dirs:
            com_num_dict[com] = len(read_file(os.path.join(com,"community_ids.txt"),"lines"))
            com_num_list.append(com_num_dict[com])

        ## sort based on size
        new_order = np.argsort(np.array(com_num_list))[::-1]
        community_dirs = np.array(community_dirs)[new_order].tolist()
        if os.path.isfile(individual_class_importance_file):
            individual_class_importance_table = np.array(read_table(individual_class_importance_file))
        else:
            individual_class_importance_table = "None"
        for com in community_dirs:
            graph_str+=add_button_head(com+" ("+str(com_num_dict[com])+" nodes)")
            graph_str+=add_community_str(base_dir, com, individual_class_importance_table)
            graph_str+="\n\t\t</div>\n"
            #graph_str+=add_br()


        #graph_str+="\n\t\t</div>\n"
        graph_str+=add_br()
        graph_str+="\n\t</div>\n"
        os.chdir('../..')
        print(os.getcwd())

    if os.path.isfile(coexpression_graph_dir+"PageRank.png"):
        graph_str+=add_h3("PageRank is a metric for how well connected a gene is in the coexpression network")
        graph_str+=add_img(coexpression_graph_dir+"PageRank.png",'Page-Rank')
    if os.path.isfile(coexpression_graph_dir+"LPR.png"):
        graph_str+=add_h3("Local PageRank is a derivative of both Louvain-modularity based community detection and PageRank. It essentially calculates a normalized PageRank within each module to quantify local connectivy throughout the graph.")
        graph_str+=add_img(coexpression_graph_dir+"LPR.png",'Local PageRank')
    graph_str+=add_h3("Here are the Z-scores for each group overlaid on the co-expression graph (red means it's enriched in that group, blue means it's low expression or not expressed):")

    ## find all of the pertinent files, and
    graph_str+=add_button_head('Z-score overlaid co-expression graphs')
    os.chdir(coexpression_graph_dir)
    extra_images = []
    for file in glob.glob('*.png'):
        if 'sample_group' in file:
            extra_images.append(file)
    extra_images = sorted(extra_images)
    os.chdir(base_dir)
    for img in extra_images:
        new,old = parse_file_name(img)
        graph_str+=add_p(new)
        graph_str+=add_img(coexpression_graph_dir+img, new)
    graph_str+="\n\t\t</div>\n"
    graph_str+=add_br()
    graph_str+="\n\t</div>\n"
    return(graph_str)


def add_img_list(list_of_images,temp_dir):
    output = ""
    for img in list_of_images:
        new,old = parse_file_name(img)
        output+=add_p(new)
        output+=add_img(temp_dir+img, new)
    return(output)


def add_genes_of_interest(base_dir):
    goi_str=""
    goi_dir = 'genes_of_interest/'
    if os.path.isdir(goi_dir):
        goi_str+=add_button_head('Genes of interest')
        goi_str+=add_p("An interesting way to use the structure of the graphs generated by MANAclust is looking at how far away all genes in your dataset are away from your genes of interest. We've shown in the MANAclust paper that there are many types of functional enrichment that correlate with how far away a gene is from another gene in the graph network. For example, close to a transcription factor are more likely to have a binding site for that transcription factor when compared to a gene that's father away from it. There is also an increased probability that two genes will encoded proteins that have a physical intereaction when those two genes are directly connected (i.e. 1-degree of separation).")
        goi_str+=add_p('below is a file containing a table that has the distance of all genes in the genome away from your gene(s) of interest.')
        goi_str+=add_file_link(goi_dir+'genes_of_interest_shortest_path_list.txt','Shortest path of all genes away from your genes of interest')
        goi_str+=add_p('Below is a heatmap of your genes of interest:')
        goi_str+=add_img(goi_dir+'genes_of_interest_subset_heatmap.png','genes of interest heatmap')

        ## collect the other images
        os.chdir(goi_dir)
        additional_images = []
        for f in glob.glob('*.png'):
            if f != 'genes_of_interest_subset_heatmap.png':
                additional_images.append(f)
        goi_str+=add_button_head("additional plots for your genes of interest")
        os.chdir(base_dir)
        goi_str+=add_img_list(sorted(additional_images),goi_dir)
        goi_str+='\n\t\t</div>\n'

        goi_str+="\n</div>\n"

    return(goi_str)


def add_autocrine_paracrine(base_dir):
    ap_dir = "autocrine_paracrine_signaling/"
    ap_str=""
    if not os.path.isdir(ap_dir):
        return(ap_str)
    else:
        ap_str+=add_button_head("Autocrine/paracrine signaling")
        ap_str+=add_p("Below are the predicted signaling networks")
        ap_str+=add_file_link(ap_dir+'all_cell_cell_interaction_summary.txt','Number of interactions between all of the groups')
        ap_str+=add_br()
        ap_str+=add_file_link(ap_dir+'all_cell_type_specific_interactions.txt','A detailed summary of each autocrine/paracrine interaction')
        ap_str+=add_br()
        ap_str+=add_file_link(ap_dir+'all_cell_type_specific_interactions_gprofiler.txt','A detailed summary of the pathways signaling across and within groups')
        ap_str+=add_br()
        ap_str+=add_file_link(ap_dir+'combined_neg_log10p_gprofiler.txt','A table of negative log10 p-values that for each pathway in each interaction. (Zero just means it did not reach signficance)')
        ap_str+=add_br()
        ap_str+=add_file_link(ap_dir+'individual_class_importance.txt','A table of the individual importance of each pathway for the given group')
        
    return(ap_str)
        #combined_neg_log10p_gprofiler.txt
        #individual_class_importance.txt


def add_gene_annotations(base_dir):
    anno_str=""
    if os.path.isfile("annotations.tsv"):
        anno_str+=add_p("Here are the annotations for your genes. You can use this in excel using v-lookup if you want to get gene symbols or definitions for any of the other files.")
        anno_str+=add_file_link("annotations.tsv","Annotation file")
    if os.path.isfile("human_orthologues.tsv"):
        anno_str+=add_br()
        anno_str+=add_file_link("human_orthologues.tsv","Human orthologues to your genes")
    anno_str+=add_br()
    return(anno_str)

def add_high_marker_genes(base_dir):
    marker_str = ""
    marker_dir = "sample_clustering_and_summary/significance/high_markers/"
    if not os.path.isfile(marker_dir+'marker_gene_annotations.tsv'):
        return("")
    else:
        marker_str+=add_button_head("Highly expressed group-specific markers")
        marker_str+=add_p("MANAclust analyzes the mean expression of each sample group and then looks for genes that meet three criteria.")
        #marker_str+=add_br()
        marker_str+=add_p("\t1) The gene is significant by 1-way ANOVA (after BH correction, alpha=0.05).")
        #marker_str+=add_br()
        marker_str+=add_p("\t2) The distance between the group with highest mean expression and second highest expression is calculated. If a gene is in the top 90th percentile of this calculation, it can make it through.")
        #marker_str+=add_br()
        marker_str+=add_p("\t3) A metric called the q-value is calculated (usually to identify outliers). The q-value is the ratio of the value calculated in number 2 compared to the range of sample group means. It's essentially what percent of the range is attributable to the distance between the highest expressing group vs the second highest expressing group.")
        #marker_str+=add_br()
        marker_str+=add_p("If all three of these criteria are, met you'll find it with an annotated group in the file below. Note that if you have several very closely related groups, there might not be many highly expressed genes that are exclusivly expressed in an individual group. In this case, you might need several markers at once to descriminate them.")
        #marker_str+=add_br()
        marker_str+=add_file_link(marker_dir+'marker_gene_annotations.tsv',"Highly expressed marker genes")
        marker_str+=add_img(marker_dir+'genes_of_interest_subset_heatmap.png',"High expression marker genes")
        marker_str+='\n</div>\n'
        return(marker_str)


###########################################################

def clean_dirs(in_dirs):
    return([d.replace('/','') for d in in_dirs])



def get_input_file_names(base_dir):
    """
    takes in the base_dir and gets you (1) a list of the categorical input files and (2) a list of the numeric input files

    example:
    input:
    get_input_file_names(base_dir)
    output:
    ["cat_1", "cat_2"], ["num_1", "num_2"]
    """
    
    ## first get the categorical ones
    cat_files = []
    for temp_file in glob.glob('*_high_info_labels.tsv'):
        cat_files.append(temp_file)
    ## strip them
    for i in range(len(cat_files)):
        cat_files[i]=cat_files[i].replace('_high_info_labels.tsv','')
    cat_files = sorted(list(set(cat_files)))

    ## now the numeric files
    num_files = []
    for temp_file in glob.glob('*_labels.tsv'):
        if not '_high_info_' in temp_file:
            #print(temp_file)
            num_files.append(temp_file)
    ## strip them
    for i in range(len(num_files)):
        num_files[i]=num_files[i].replace('_labels.tsv','')
    num_files = sorted(list(set(num_files)))

    cat_files = clean_dirs(cat_files)
    num_files = clean_dirs(num_files)
    return(cat_files, num_files)

###########################################################


def add_single_cat_feat(dataset):
    cat_feat = ""
    cat_feat += add_button_head(dataset+"'s feature selection")
    cat_feat += add_h2(dataset+"'s affinity matrix when calculated withOUT feature selection:")
    cat_feat += add_img(dataset+"_without_feature_selection.png",dataset+"w.o_feat")
    cat_feat += add_h2(dataset+"'s affinity matrix when calculated WITH feature selection:")
    cat_feat += add_img(dataset+"_high_info_with_feature_selection.png",dataset+"w._feat")
    cat_feat += '\n</div>\n'
    return(cat_feat)


def add_single_num_feat(dataset):
    num_feat = ""
    num_feat += add_button_head(dataset+"'s feature selection")
    num_feat += add_h2(dataset+"'s affinity matrix when calculated withOUT feature selection:")
    num_feat += add_img(dataset+"_without_feature_selection.png",dataset+"w.o_feat")
    num_feat += add_h2(dataset+"'s affinity matrix when calculated WITH feature selection:")
    num_feat += add_img(dataset+"_with_feature_selection.png",dataset+"w._feat")
    ## now heatmaps
    num_feat += add_h2("A heatmap for "+dataset+" withOUT feature selection:")
    num_feat += add_img(os.path.join(dataset,dataset+"_sample_of_all_features.png"),dataset+"w.o_feat_heat")
    num_feat += add_h2("A heatmap for "+dataset+" WITH feature selection:")
    num_feat += add_img(os.path.join(dataset,dataset+"_selected_features.png"),dataset+"w._feat_heat")
    num_feat += '\n</div>\n'
    return(num_feat)


def add_tab_header(section_name, default_open=False):
    out_head = "<div id='"+section_name+"' class='w3-container w3-border city'"
    if not default_open:
        out_head += ' style="display:none"'
    else:
        out_head += ' style="display: block;"'
    out_head += '>'
    return(out_head)


def add_feat_select(base_dir, cat_files, num_files):
    feat_str = ""
    feat_str += add_tab_header("Feature Selection", default_open=False)
    #feat_str+=add_button_head("Feature selection results")
    feat_str+=add_p("\nOne of the first things that MANAclust does is feature selection. Often times, for unsupervised clustering, one doesn't know which features will be imporant and which ones are just randomly adding unpatterned variance. MANAclust tackles this problem for both categorical datasets and numeric datasets, which are listed below:")
    if len(cat_files)>0:
        feat_str+=add_button_head("Categorical datasets")
        ## add the documentation for each of the categorical datasets
        feat_str+=add_p("\nCategorical feature selection is based on the pairwise feature-feature mutual information, when compared to a shuffled version of the dataset comparing your real data to a null distribution. Below are depictions of the sample-sample affinity matrix as calculated through the sum of the log-loss or categorical cross entropy between subjects. There is one affinity matrix showing what the dataset looks like before feature selection, then another that shows the affinity matrix after.")
        for cat_file in cat_files:
            feat_str+=add_single_cat_feat(cat_file)
        feat_str+='\n</div>\n'
    if len(num_files)>0:
        feat_str+=add_button_head("Numeric datasets")
        ## add the documentation for each of the numeric datasets
        feat_str+=add_p("\nNumeric feature selection is based on the pairwise feature-feature mutual information, when compared to a shuffled version of the dataset comparing the distribution of negative Spearman Rhos to your actual dataset, selecting features that have notably more negative correlatoins than expected by chance. Below are depictions of the sample-sample affinity matrix as calculated through the normalized negative squared Eculidean distance across samples. There is one affinity matrix showing what the dataset looks like before feature selection, then another that shows the affinity matrix after. Also included below are heatmaps of the datasets pre- and post- feature selection.")
        for num_file in num_files:
            feat_str+=add_single_num_feat(num_file)
        feat_str+='\n</div>\n' 


    #feat_str+=add_br()
    feat_str+=add_button_head("Final Merged Datasets")
    feat_str+=add_p("MANAclust calculates a final merged affinity matrix through normalizing each of the input & taking the missing-value compatible mean of all affinity matrices. In this way, subtle differences that are consistent across datasets will re-inforce each other, while large differences in a single dataset will still split off a new final cluster. Similarly, strong concordance in a single dataset won't over-power consistent differences across other datasets. Imporantly, because it's a missing-value compatible calculation, you can use it in a real-word context where this is the norm rather than the exception!")+add_p("\n")+add_p("Below we show the merged affinity matrix with and without feature selection. We also show a 2D projection (using the UMAP) algorithm of the affinity matrices.")
    feat_str+=add_h2("Below is the final affinity matrix and UMAP projection if we DIDN'T do feature selection")
    feat_str+=add_p("Merged affinity matrix without feature selection:")
    feat_str+=add_img("final_affinity_matrix_without_feature_selection.png","final_affinity_matrix_without_feature_selection")
    feat_str+=add_p("2D projection of merged affinity matrix without feature selection:")
    feat_str+=add_img("final_affinity_matrix_umap_without_feature_selection_clustering.png","final_affinity_matrix_umap_without_feature_selection_clustering")
    feat_str+=add_h2("Below is the final affinity matrix if we DO use feature selection")
    feat_str+=add_p("Merged affinity matrix WITH feature selection:")
    feat_str+=add_img("final_affinity_matrix_with_feature_selection.png","final_affinity_matrix_with_feature_selection")
    feat_str+=add_p("2D projection of merged affinity matrix WITH feature selection:")
    feat_str+=add_img("final_affinity_matrix_umap_with_feature_selection_clustering.png","final_affinity_matrix_umap_with_feature_selection_clustering")
    #feat_str+=add_br()
    feat_str+='\n</div>\n'

    feat_str+='\n</div>\n'

    feat_str+='\n</div>\n'
    #feat_str+=add_br()
    return(feat_str)


def add_final_cluster_sections(base_dir, cat_files, num_files, with_feat = True):
    group_annotatoins
    if with_feat:
        image_ender = "with_feature_selection.png"
        label_ender = "_labels.tsv"
    else:
        image_ender = "without_feature_selection.png"
        label_ender = "_labels_full.tsv"
    ## first do the final clustering results

    ## first do the cat files

    ## then do the num files

    ## then add the images
    return()

def add_group_annotations(base_dir, cat_files, num_files):
    ##
    group_str = ""
    group_str+=add_tab_header("Sample Groups")
    #group_str+=add_button_head("Sample groups - both the final clusters & the consensus groups from the input datasets")
    #group_str+=add_br()
    group_str += add_h2("Final clusters:")
    group_str += add_p("Below are the annotatoins for each sample/subject and their ")
    group_str += add_file_link("sample_final_cluster_and_consensus_group_annotations.tsv", "Each sample's final cluster and data-set level consensus group IDs")
    group_str += add_h2("Final cluster to Consensus Group Mappings:")
    group_str += add_p("Every final cluster maps to a unique combination of the consnesus groups from your input datasets. Below is the table that maps from the final clusters to the consensus groups for the input datasets. If you're looking for some pretty plots, there are some of those in the feature selection tab that plots the samples/subjects as affinity matrices and 2D UMAP projections of those affinity matrices. Note that if a subject was missing an ome, it of course won't have a consensus group label for that dataset, instead it will say 'NA'.")
    group_str += add_file_link("final_clusters_to_consensus_group_annotations.tsv","File with mappings between final clusters and consensus groups.")
    group_str += add_table(read_table(os.path.join("final_clusters_to_consensus_group_annotations.tsv"))) + add_br()

    group_str+='\n</div>\n'
    group_str+='\n</div>\n'
    #group_str+=add_br()
    return(group_str)




def add_between_con_groups(base_dir, cat_files, num_files):
    ##
    btw_con_group_str = ""
    btw_con_group_str+=add_button_head("What is the relationship between each of the consensus groups?")
    ## first add a paragraph describing what this means

    ## then go through each of the 



    btw_con_group_str+='\n</div>\n'
    btw_con_group_str+=add_br()
    return(btw_con_group_str)

def add_stats_results(dataset_dir, dataset_name, consensus=False):
    if consensus:
        leader = os.path.join(dataset_dir, "consensus_groups", dataset_name)
    else:
        leader = os.path.join(dataset_dir, dataset_name)
    temp_str = ""
    temp_str += add_file_link(leader+"_global_statistical_differences.tsv","\nAll global statistics")
    temp_str += add_p("\n")
    temp_str += add_file_link(leader+"_all_significant_postHocs.tsv","\nSignificant pair-wise post-hoc tests")
    temp_str += add_p("\nThe total number of signficant post-hoc differences across the groups:")
    temp_str += add_img(leader+"_number_different_across_groups.png", "num_dif")
    return(temp_str)


def add_single_dataset_stats(dataset_dir, dataset_name):
    leader = dataset_name.replace('/','')
    #print(leader)
    single_dataset_str=""
    ## add button for dataset
    single_dataset_str+=add_button_head(str(dataset_dir))
    #### add button for final groups
    single_dataset_str+=add_button_head("Stats for '"+str(dataset_dir)+"' based on the final clusters")
    #### add paragraph about the next results that are testing the final groups
    single_dataset_str+=add_p("\nThese are the statistics for '"+str(dataset_dir)+"' based on the final clusters. Note that there could be some groups that have essentially no difference. When this happens, it's becuase these groups were essentially the same by this dataset (and likely came out in the same consensus group), but they were orthogonal to each other by a different dataset. If you're interested in seeing the consensus group results, they're on the next button down. That tab also shows how many significant differences there were between the final clusters & the statistics that led to the creation of the consensus groups.")
    #### add the final group stats for this ome
    single_dataset_str+=add_stats_results(dataset_dir, leader, consensus = False)



    ## add br
    single_dataset_str+='\n</div>\n'

    single_dataset_str+=add_button_head("Stats for '"+str(dataset_dir)+"' based on the consensus groups")
    #### add button for consensus group analysis
    single_dataset_str+=add_p("\nThese are the statistics for '"+str(dataset_dir)+"' based on the consensus groups for this ome. We also have below the statistics of the final-groups that led to some of them (perhaps) being merged into single consensus groups.")

    #### first add the images and descriptions about how the consensus groups are made
    single_dataset_str+=add_p("\n\tThis shows each of the final clusters & the probability that their members came from the same distribution within this ome. This was calculated by comparing the within-final-group to across final-group affinities for this dataset by ANOVA, then Benjamini-Hochberg correcting this probability for all of these comparisons. This gives us a final probability that these subjects came from the same distribution. This matrix was then transformed into a p-value weighted network graph, that was subjected to Louvain-modularity analysis to determine the consensus groups for this dataset.")
    single_dataset_str+=add_img(os.path.join(dataset_dir,leader+"_affinity_BH_p_val_matrix.png"), "BH_p_val")

    single_dataset_str+=add_p("\n\tThis is the negative log10 p-values of the probabilities from above.")
    single_dataset_str+=add_img(os.path.join(dataset_dir,leader+"_affinity_neg_log10_BH_p_val_matrix.png"), "BH_p_val")

    single_dataset_str+=add_p("\n\tThis shows the F statistic for the ANOVAs calculated above.")
    single_dataset_str+=add_img(os.path.join(dataset_dir,leader+"_affinity_F_statistic_matrix.png"), "F_stat")

    single_dataset_str+=add_p("\n\tBelow are the global and pairwise statistics for each of the consensus group level comparisons for this dataset:")
    #### then add the consensus group folder
    single_dataset_str+=add_stats_results(dataset_dir, leader, consensus = True)

    single_dataset_str+='\n</div>\n'

    single_dataset_str+='\n</div>\n'
    single_dataset_str+=add_br()    
    return(single_dataset_str)


def add_sig_dif_between_groups(base_dir):
    ##
    sig_dif_str = ""
    sig_dif_str+=add_tab_header("Significant Differences")
    #sig_dif_str+=add_button_head("What are the significant differences between these groups?")
    ## add a paragraph about what each 
    sig_dif_str+=add_p('\n')+add_p("For each of your input datasets (including the 'test' datasets that weren't used for clustering - if there were any), below are the traditional statistics comparing each of the 'final clusters' to each other as well as the within-dataset consensus group differences. This can help find biomarkers for consensus groups or final clusters.")
    sig_dif_str+=add_p('\n')+add_p("For all catagorical datasets, we first do a global chi-square test in which we look at the frequency of each category relative to each of the final or consensus groups. These are then Benjamini-Hochberg FDR corrected. Anything that was signficant (default is FDR-q<0.001, set by the '-alpha' parameter) is then passed through to do pairwise Chi-squares, again BH-FDR corrected. Note that in mixed numeric/categorical datasets that were fed in as categorical datasets, the numeric or mixed variables are converted to binned values & treated as categrical - this can be seen as similar to doing a maximum-informatoin style test on numeric datasets, except it's just chi-square.")
    sig_dif_str+=add_p('\n')+add_p("For the numeric datasets, a similar procedure is done as with the above categorical datasets, except they're tested by ANOVAs instead of chi-square.")

    ## make a button for each input dataset & list out & describe the files/images
    dataset_dirs = []
    for temp_path in glob.glob('*'):
        if os.path.isdir(temp_path):
            if temp_path not in ["test_ome_differences_by_consensus_groups", "between_ome_consensus_group_comparisons"]:
                dataset_dirs.append(temp_path)
    dataset_dirs = clean_dirs(sorted(dataset_dirs))
    for temp_dir in dataset_dirs:
        sig_dif_str+=add_single_dataset_stats(temp_dir,temp_dir)

    sig_dif_str+='\n</div>\n'
    sig_dif_str+='\n</div>\n'
    #sig_dif_str+=add_br()
    return(sig_dif_str)


def add_single_test_ome(test_ome_dir):
    test_ome_dir = os.path.join("test_ome_differences_by_consensus_groups",test_ome_dir)
    test_ome_name = os.path.basename(os.path.normpath(test_ome_dir))
    dataset_dirs = []
    for temp_path in glob.glob(os.path.join(str(test_ome_dir))+os.path.sep+'*'):
        #print(temp_path,'in',test_ome_dir)
        if os.path.isdir(temp_path):
            dataset_dirs.append(temp_path)
            #print("\t\t",dataset_dirs[-1])
    
    out_str = ""
    for temp_sub_dataset in dataset_dirs:
        temp_name = os.path.basename(os.path.normpath(temp_sub_dataset))
        out_str+=add_button_head(temp_name)
        #print("\n\n",temp_sub_dataset)
        out_str+=add_stats_results(temp_sub_dataset, test_ome_name)
        out_str+='\n</div>\n'

    return(out_str)



def add_test_omes(base_dir):
    #print("\ndoing test omes")
    #test_dir = os.path.join(base_dir,"test_ome_differences_by_consensus_groups")
    test_dir = "test_ome_differences_by_consensus_groups"
    test_v_con_str = ""
    test_v_con_str+= add_tab_header("Test Ome Significant Differences")
    if not os.path.isdir(test_dir):
        test_v_con_str += add_h2("No test omes were used in this analysis")
        test_v_con_str+='\n</div>\n'
        return(test_v_con_str)
    test_v_con_str+=add_h2("How do the consensus groups based on your input datasets differ in the test datasets?")
    test_v_con_str+= add_p("In these statistical comparisons, we assessed each variable in each of the test datasets based on their group membership based on all of the consensus groups from the other omes. In this way, you can find out what 'If we take the consensus groups from ome X, what would be the sinificant differences in ome Y?'")

    dataset_dirs = []
    for temp_path in glob.glob('test_ome_differences_by_consensus_groups'+os.path.sep+'*'):
        if os.path.isdir(temp_path):
            dataset_dirs.append(temp_path.replace("test_ome_differences_by_consensus_groups/",''))


    for dataset in dataset_dirs:
        ## add button
        #print('\t',dataset)
        test_v_con_str+=add_button_head(dataset)
        test_v_con_str+=add_p("\nBelow are how the above test ome was different when using the consensus groups as defined by the following input datasets used for clustering:")
        test_v_con_str+=add_single_test_ome(dataset)
        test_v_con_str+='\n</div>\n'


    test_v_con_str+='\n</div>\n'
    test_v_con_str+='\n</div>\n'

    #test_v_con_str+=add_br()
    return(test_v_con_str)


def add_pre_button_header(base_dir):
    pre_button_header = ""
    ## add a short paragraph about MANAclust, what it does & what the results are below.


    ## add the final group annotatoins

    return(pre_button_header)


def add_final_level_cross_ome_comparison(final_dir, final_name):
    final_layer_cross_ome_str = ""
    ## add the button
    final_layer_cross_ome_str += add_button_head(" -- VS -- ".join(final_name.split("_vs_")))
    ## first add the contingency table
    final_layer_cross_ome_str += add_h3("Contingency table:")
    final_layer_cross_ome_str += add_img(os.path.join(final_dir,"contingency_table.png"),"cont_table")
    ## then residuals
    final_layer_cross_ome_str += add_h3("Chi-squared adjusted residuals:")
    final_layer_cross_ome_str += add_p("Chi-squared adjusted residuals is kind of like a 'Z-score' for chi-quared analyses. So a high positive the number in the below matrix means that we observed far more samples/subjects here than we would expect by chance. A very negative number means that these two consensus groups were unexpectedly discordant.")
    final_layer_cross_ome_str += add_img(os.path.join(final_dir,"chi_adjusted_residuals.png"),"chi_adj_res")
    ## then p-values
    final_layer_cross_ome_str += add_h3("BH-FDR corrected p-values:")
    final_layer_cross_ome_str += add_img(os.path.join(final_dir,"adjusted_p_values.png"),"p-vals")
    ## close the button
    final_layer_cross_ome_str +='\n</div>\n'
    return(final_layer_cross_ome_str)


def add_subdir_cross_ome(in_base):
    subdir_str = ""
    dataset_dirs = []
    comparison_names = []
    print('\n\nDoing',in_base+os.path.sep+'*')
    for temp_path in glob.glob(in_base+os.path.sep+'*'):
        #print('\tChecking:',temp_path)
        if os.path.isdir(temp_path):
            dataset_dirs.append(temp_path)
            temp_name = os.path.basename(os.path.normpath(temp_path))
            comparison_names.append(temp_name)
    for i in range(len(dataset_dirs)):
        print('\t',comparison_names[i])
        subdir_str += add_final_level_cross_ome_comparison(dataset_dirs[i], comparison_names[i])
    return(subdir_str)


def add_cross_ome_stats():
    cross_ome_str=""
    base = "between_ome_consensus_group_comparisons"
    sub_dirs = ["between_cluster_and_test_omes",
                "between_clustering_omes",
                "between_test_omes"]
    cross_ome_comparison_dirs = [os.path.join(base,temp_dir) for temp_dir in sub_dirs]
    ## first make the top button
    cross_ome_str += add_tab_header("Cross Ome Concordance")
    #cross_ome_str += add_button_head("How concordant/discordant are consensus groups with each other?")
    ## add a paragraph describing what's below
    cross_ome_str += add_p("\nHere, we look at the subjects that lie in each consensus group from each of the datasets & look for how concordant/discordant the members from each of these datasets are. For example, take the hypothetical that two input datasets essentially carry the same information (ie: subjects that are similar to each other in one dataset are also similar to each other in the other dataset); the subjects in dataset #1, consensus cluster #1 might be the same subjects that are present in dataset #2 consnesus cluster #3. We assess this concordance/discordance through chi-squared tests. Below are the Chi-square results for the comparison of consensus groups from the input datasets used for clustering, input vs test datasets, and test vs test datasets.")+add_p("\n")+add_p("The contingency tables list the number of subjects belonging to each of the pairwise groups.")+add_p("\n")+add_p("The adjusted residuals table can be interpreted like Z-scores for the contingency table, indicating how over- or under- abundant a given cell in the matrix is.")+add_p("\n")+add_p("These are then used to calculate the cell-wise p-values, which are BH-FDR corrected for multiple comparisons.")
    cross_ome_str += add_file_link(os.path.join(base,"all_combined_cross_ome_concordance_discordance_statistics.tsv"),"Here is a summary of all of the datasets & their consensus group concordance/discordance.")
    cross_ome_str += add_p("This includes the 'adacency list' with Bayesian statistics showing the probability of being in one consensus group of from one data type, given that you're a member \
        of another consensus group in a different datatype. For example: P(A|B) means the probability of being in group A, given that you're a member of group B. \
     In this scenario, this might equate to: What's the probability of being in Transcriptome consensus group 0, given that you're in Methylome consensus group 3? This probability can range from 0, to 1, \
     In the before example, if P(Transcriptome||consensus_group_1 | Methylome||consensus_group_3) = 1, that means that ALL of the Methylome||consensus_group_3 members were also members of Transcriptome||consensus_group_1. This can be loosly interpreted as direcitonality, \
     but in reality, you'd need an intervention/experiment to prove it.")
    cross_ome_str += add_p("In each of the sub-tabs below, there are subsets of the above combined results.")
    cross_ome_str += add_p("You can also use this file as a 'Bayesian-like network' (although, it's a little different) - it could be very useful to import \
        this into [cytoscape](https://cytoscape.org/download.html), and create some filters to *suggest* causal relationships. \
        You can do this, by setting up some filters, like: ")
    cross_ome_str += add_p("\t1) keeping only edges that are signficant by BH-corrected p-values from the Chi-squared")
    cross_ome_str += add_p("\t2) keeping only edges that have a positive adjusted residuals (this means that these two groups are positively concordant, rather than discordant)")
    cross_ome_str += add_p("\t3) keeping edges that have a P(group2|group1)-P(group1|group2) value > -0.15 (or something like that). This allows for some 'ties' in directionality, \
        where the probility of going in one direction is fairly similar to going in the other direction.")
    cross_ome_str += add_p("I also like to plot the edge width based on the P(group2|group1) value as this shows the magnitude of the effect.")
    ## second
    cross_ome_str += add_button_head("Input clustering datasets -- VS -- other input clustering datasets")
    cross_ome_str += add_br()
    cross_ome_str += add_file_link(os.path.join(cross_ome_comparison_dirs[1],"cross_ome_concordance_discordance_statistics.tsv"),"Here is a summary of all of the clustering datasets & their consensus group concordance/discordance")
    cross_ome_str += add_br()
    cross_ome_str += add_subdir_cross_ome(cross_ome_comparison_dirs[1])
    cross_ome_str += "\n</div>\n"
    ## first
    cross_ome_str += add_button_head("Input clustering datasets -- VS -- those used for testing")
    cross_ome_str += add_br()
    cross_ome_str += add_file_link(os.path.join(cross_ome_comparison_dirs[0],"cross_ome_concordance_discordance_statistics.tsv"),"Here is a summary of all of the clustering vs test datasets & their consensus group concordance/discordance")
    cross_ome_str += add_br()
    cross_ome_str += add_subdir_cross_ome(cross_ome_comparison_dirs[0])
    cross_ome_str += "\n</div>\n"
    ## third
    cross_ome_str += add_button_head("Test dataset consnensus groups -- VS -- other test datasets")
    cross_ome_str += add_br()
    cross_ome_str += add_file_link(os.path.join(cross_ome_comparison_dirs[2],"cross_ome_concordance_discordance_statistics.tsv"),"Here is a summary of all of the test datasets & their consensus group concordance/discordance")
    cross_ome_str += add_br()
    cross_ome_str += add_subdir_cross_ome(cross_ome_comparison_dirs[2])
    cross_ome_str += "\n</div>\n"
    cross_ome_str += "\n</div>\n"

    ## close this sub-set
    cross_ome_str += "\n</div>\n"
    return(cross_ome_str)


def add_summary():
    summary_str = ""
    summary_str += add_tab_header("Summary",default_open=True)
    summary_str += add_h2("Welcome to the MANAclust output walkthrough!")
    summary_str += add_p("MANAclust works in several stages, which each has a tab located above. Here's a brief description of what each of those tabs holds and what it means.")
    summary_str += add_h3("Feature Selection")
    summary_str += add_p("In brief, MANAclust took your input numeric and categorical datasets, found which of the features were adding structure to those datasets, \
        converted the datastes into cross-subject affinity matrices (how similar subjects are to each other), merged them together and clustered them using affinity propagation. \
        This tab has plots of the affinity matrices for each ome with and without feature selection \
        as well as plots showing the final clusters.")
    summary_str += add_h3("Sample Groups")
    summary_str += add_p("Once the clustering has been completed, the final clusters (which used all of the input datasets) as well as the 'consensus groups' which are \
     the within-dataset groups are annotated. The files and tables in this tab annotate which subject belongs to which final cluster as well as the consensus groups within each dataset.")
    summary_str += add_h3("Significant Differences")
    summary_str += add_p("For each dataset, MANAclust went through and calculated signficant differences of all of the features of each dataset, comparing (1) final clusters to each other, and \
        (2) consensus groups for the given ome.")
    summary_str += add_h3("Test Ome Significant Differences")
    summary_str += add_p("Similar to the above described significant differences, except, in this tab, we assessed signficant differences in the held-out test omes (if any were used),\
     and ran comparisons for the consensus groups of the other input datasets. \
     So for example, if you performed clustering using the methylome and transcriptome, but had a dataset A (categorical or numeric) that you wanted to use, not for clustering, but just to look at the differences in \
     based on the clustering results from methylome & transcriptome - this is where you'd find those statistical comparisons.")
    summary_str += add_h3("Cross Ome Concordance: How do these different datasets weave together?")
    summary_str += add_p("One imprtant aspect of multi-omics is determining whether or not your input datasets had shared components of variance, or of they were actually orthogonal to each other, providing \
        completely different information. MANAclust went through each one of your consensus groups from each pair of input datasets & assessed whether any of these consensus groups were significantly concordant with each other \
        or significantly discordant. For example, in Dataset1, if all of the subjects in consensus group 0, were all the same people in Dataset2 consensus group 3, you'd have a signficant result here. This is measured by \
        Chi-squared analysis, using the Chi-squared adjusted residuals to show over-abundance (positive value), or discordance between two groups (negative value).")
    summary_str += add_p("Furthermore, we also automated generation of Bayesian statistics. This lets you build a Bayesian-like network to hypothesize potential causal relationships across omes. Check out the tab for more details!")
    summary_str += "\n</div>\n"
    return(summary_str)

###########################################################

def make_webpage(base_dir):
    #out_file = os.path.join(base_dir, "MANAclust_summary.html")
    out_file = "MANAclust_summary.html"
    os.chdir(base_dir)


    ## first add the generic heading
    out_web = """<!DOCTYPE html>
    <html>

    <title>MANAclust Results</title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" href="https://www.w3schools.com/w3css/4/w3.css">

    <head>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <style>
    .collapsible {
        background-color: #777;
        color: white;
        cursor: pointer;
        padding: 18px;
        width: 100%;
        border: none;
        text-align: left;
        outline: none;
        font-size: 15px;
    }

    .active, .collapsible:hover {
        background-color: #555;
    }

    .content {
        padding: 0 18px;
        display: none;
        overflow: hidden;
        background-color: #f1f1f1;
    }
    table {
      font-family: arial, sans-serif;
      border-collapse: collapse;
      width: 100%;
    }

    td, th {
      border: 1px solid #dddddd;
      text-align: left;
      padding: 8px;
    }

    tr:nth-child(even) {
      background-color: #dddddd;
    }
    </style>
    </head>

    <body>





    <div class="w3-container w3-teal">
       <h1>MANAclust Results</h1>
    </div>


    <div class="w3-container">
      
      <div class="w3-bar w3-black">
        <button class="w3-bar-item w3-button tablink w3-red" onclick="openCity(event,'Summary')">Summary</button>
        <button class="w3-bar-item w3-button tablink" onclick="openCity(event,'Feature Selection')">Feature Selection</button>
        <button class="w3-bar-item w3-button tablink" onclick="openCity(event,'Sample Groups')">Sample Groups</button>
        <button class="w3-bar-item w3-button tablink" onclick="openCity(event,'Significant Differences')">Significant Differences</button>
        <button class="w3-bar-item w3-button tablink" onclick="openCity(event,'Test Ome Significant Differences')">Test Ome Significant Differences</button>
        <button class="w3-bar-item w3-button tablink" onclick="openCity(event,'Cross Ome Concordance')">Cross Ome Concordance</button>
      </div>
      
    </div>


    """

    ## get all of the input datasets
    cat_files, num_files = get_input_file_names(base_dir)

    ## print out the categorical ones
    if len(cat_files)>0:
        print("found",len(cat_files), "categorical files")
        for c in cat_files:
            print("\t",c)

    ## print out the numeric ones
    if len(num_files)>0:
        print("found",len(num_files), "numeric files")
        for c in num_files:
            print("\t",c)

    ## add the bulk of the content
    out_web+=add_pre_button_header(base_dir)
    out_web+=add_summary()
    out_web+=add_feat_select(base_dir, cat_files, num_files)
    out_web+=add_group_annotations(base_dir, cat_files, num_files)
    out_web+=add_sig_dif_between_groups(base_dir)
    out_web+=add_test_omes(base_dir)
    out_web+=add_cross_ome_stats()

    out_web+="""


    <script>
    var coll = document.getElementsByClassName("collapsible");
    var i;

    for (i = 0; i < coll.length; i++) {
        coll[i].addEventListener("click", function() {
            this.classList.toggle("active");
            var content = this.nextElementSibling;
            if (content.style.display === "block") {
                content.style.display = "none";
            } else {
                content.style.display = "block";
            }
        });
    }
    </script>


    <script>
    function openCity(evt, cityName) {
      var i, x, tablinks;
      x = document.getElementsByClassName("city");
      for (i = 0; i < x.length; i++) {
        x[i].style.display = "none";
      }
      tablinks = document.getElementsByClassName("tablink");
      for (i = 0; i < x.length; i++) {
        tablinks[i].className = tablinks[i].className.replace(" w3-red", "");
      }
      document.getElementById(cityName).style.display = "block";
      evt.currentTarget.className += " w3-red";
    }
    </script>




    </body>
    </html>



    """


    out_web = out_web.replace('//','/')
    out_web = out_web.replace(base_dir,'')


    make_file(out_web,out_file)
    
    return()

###########################################################

if __name__ == "__main__":
    ####################
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-base_dir','-in','-i','-input',
        dest='base_dir',
        type=str)
    args = parser.parse_args()
    ####################
    base_dir = os.path.join(args.base_dir)


    make_webpage(base_dir)


"""
A library of functions that are used in the Scripts for PyMiner's auxilary scripts.  Reduced code duplication and makes
maintenance easier. """

import pickle
import os
import numpy as np
import networkx as nx
import community
from subprocess import check_call, Popen, PIPE

def process_dir(in_dir):
	## process the output dir
	if in_dir[-1]!='/':
		in_dir+='/'
	if not os.path.isdir(in_dir):
		os.makedirs(in_dir)
	return(in_dir)


def run_cmd(in_message, com=True, stdout=None):

    print('\n', " ".join(in_message), '\n')
    if stdout:
        with open(stdout, 'w') as out:
            process = Popen(in_message, stdout=PIPE)
            while True:
                line = process.stdout.readline().decode("utf-8")
                out.write(line)
                if line == '' and process.poll() is not None:
                    break
    if com:
        Popen(in_message).communicate()
    else:
        check_call(in_message)


def read_file(temp_file, lines_o_raw='lines', quiet=False):
    """ basic function library """
    lines = None
    if not quiet:
        print('reading', temp_file)
    file_handle = open(temp_file, 'r')
    if lines_o_raw == 'lines':
        lines = file_handle.readlines()
        for i, line in enumerate(lines):
            lines[i] = line.strip('\n')
    elif lines_o_raw == 'raw':
        lines = file_handle.read()
    file_handle.close()
    return lines


def make_file(contents, path):
    file_handle = open(path, 'w')
    if isinstance(contents, list):
        file_handle.writelines(contents)
    elif isinstance(contents, str):
        file_handle.write(contents)
    file_handle.close()


# def flatten_2D_table(table, delim):
#     # print(type(table))
#     if str(type(table)) == "<class 'numpy.ndarray'>":
#         out = []
#         for i, row in enumerate(table):
#             out.append([])
#             for j, cell in enumerate(row):
#                 try:
#                     str(cell)
#                 except ValueError:
#                     print(cell)
#                 else:
#                     out[i].append(str(cell))
#             out[i] = delim.join(out[i]) + '\n'
#         return out
#     else:
#         for i, row in enumerate(table):
#             for j, cell in enumerate(row):
#                 try:
#                     str(row)
#                 except ValueError:
#                     print(row)
#                 else:
#                     table[i][j] = str(cell)
#             table[i] = delim.join(row) + '\n'
#         return table

def flatten_2D_table(table, delim):
    # print(type(table))
    if str(type(table)) == "<class 'numpy.ndarray'>":
        out = []
        for i in range(0, len(table)):
            out.append([])
            for j in range(0, len(table[i])):
                try:
                    str(table[i][j])
                except:
                    print(table[i][j])
                else:
                    out[i].append(str(table[i][j]))
            out[i] = delim.join(out[i]) + '\n'
        return out
    else:
        for i in range(0, len(table)):
            for j in range(0, len(table[i])):
                try:
                    str(table[i][j])
                except:
                    print(table[i][j])
                else:
                    table[i][j] = str(table[i][j])
            table[i] = delim.join(table[i]) + '\n'
        # print(table[0])
        return table


def strip_split(line, delim='\t'):
    return line.strip('\n').split(delim)


def make_table(lines, delim, range_min=0):
    for i in range(range_min, len(lines)):
        lines[i] = lines[i].strip()
        lines[i] = lines[i].split(delim)
        for j in range(range_min, len(lines[i])):
            if i!=0 or j!=0:
                try:
                    float(lines[i][j])
                except ValueError:
                    lines[i][j] = lines[i][j].replace('"', '')
                else:
                    lines[i][j] = float(lines[i][j])
    return lines


def get_file_path(in_path):
    in_path = in_path.split('/')
    in_path = in_path[:-1]
    in_path = '/'.join(in_path)
    return in_path + '/'


def read_table(file, sep='\t'):
    return make_table(read_file(file, 'lines'), sep)


def write_table(table, out_file, sep='\t'):
    make_file(flatten_2D_table(table, sep), out_file)


def import_dict(file_handle):
    file_handle = open(file_handle, 'rb')
    data = pickle.load(file_handle)
    file_handle.close()
    return data


def save_dict(data, path):
    file_handle = open(path, 'wb')
    pickle.dump(data, file_handle)
    file_handle.close()


# def get_sample_k_lists(group_numeric_vector, total_groups = None):
#     if total_groups == None:
#         total_groups = max(group_numeric_vector)+1
#     print(total_groups)
#     group_index_lists = [[] for i in range(total_groups)]
#     for i in range(0,len(group_numeric_vector)):
#         temp_group = group_numeric_vector[i]
#         group_index_lists[temp_group].append(i)
#     print("\n\ngroup_index_lists")
#     print(group_index_lists)
#     return(group_index_lists)


def get_sample_k_lists(group_numeric_vector, total_groups = None):
    temp_group_numeric_vector = []
    for num in group_numeric_vector:
        try:
            temp_group_numeric_vector.append(int(num))
        except:
            temp_group_numeric_vector.append(num)
    group_numeric_vector = temp_group_numeric_vector
    if total_groups == None:
        all_vect = list(set(group_numeric_vector))
        numeric_vect = []
        for entry in all_vect:
            #print(type(entry),entry)
            if type(entry)==int:
                numeric_vect.append(entry)
        total_groups = max([len(numeric_vect),max(numeric_vect)+1])
    # print(total_groups)
    group_index_lists = [[] for i in range(total_groups)]
    for i in range(0,len(group_numeric_vector)):
        if type(group_numeric_vector[i]) ==  int or type(group_numeric_vector[i]) == float:

            temp_group = group_numeric_vector[i]
            group_index_lists[temp_group].append(i)
    # print("\n\ngroup_index_lists\n\n")
    # print(group_index_lists)
    return(group_index_lists)


def convert_vect_to_mi_bins(in_vect):
    out_vect = np.array(in_vect, dtype = str)
    ## first get all the unique values
    all_vals = list(set(list(in_vect)))
    ## next get all the numeric convertable values
    ## or return the original vect if there are none
    all_num_vals = []
    all_num_idxs=[]
    for i in range(len(in_vect)):
        try:
            float(in_vect[i])
        except:
            pass
        else:
            temp_val = float(in_vect[i])
            if not np.isnan(temp_val):
                all_num_vals.append(temp_val)
                all_num_idxs.append(i)
    ## if there aren't enough numeric values to warrant conversion, then don't
    if len(all_num_vals) < 10:
        return in_vect
    ## calculate the number of bins
    bin_num = max([2,int(np.sqrt(len(all_num_vals)/5))])
    ## min-max linear normalize the vector
    all_num_vals = np.array(all_num_vals)
    all_num_vals -= np.min(all_num_vals)
    all_num_vals = all_num_vals / np.max(all_num_vals)
    ## calculate the bin interval
    bins = np.arange(bin_num)/bin_num
    digitized_num_vals = np.digitize(all_num_vals, bins)
    print(digitized_num_vals)
    ## now update the original vector to return it
    for i in range(0,len(all_num_idxs)):
        original_idx = all_num_idxs[i]
        new_val = digitized_num_vals[i]
        out_vect[original_idx] = str(new_val)
    return out_vect


def digitize_for_max_info(in_mat):
    ## takes in a full input matrix & digitizes the numeric values of the colums
    in_mat = np.array(in_mat,dtype=str)
    ncol = np.shape(in_mat)[1]
    nrow = np.shape(in_mat)[0]
    for i in range(1,ncol):
        in_mat[1:nrow,i] = convert_vect_to_mi_bins(in_mat[1:nrow,i])
    return in_mat


## this function was adopted from emre's stackoverflow answer found here:
## https://stackoverflow.com/questions/7450957/how-to-implement-rs-p-adjust-in-python
def correct_pvalues_for_multiple_testing(pvalues, correction_type = "Benjamini-Hochberg"):                
    """                                                                                                   
    consistent with R - print correct_pvalues_for_multiple_testing([0.0, 0.01, 0.029, 0.03, 0.031, 0.05, 0.069, 0.07, 0.071, 0.09, 0.1]) 
    """
    from numpy import array, empty
    pvalues = array(pvalues)

    ## convert to linear if needed
    #print(pvalues)
    if len(np.shape(pvalues)) > 1:
        needs_reshaping = True
        original_shape = np.shape(pvalues)
        new_shape = pvalues.size
        #print(new_shape)
        pvalues = pvalues.reshape((new_shape))
        #print(pvalues)
    else:
        needs_reshaping = False

    n = int(pvalues.shape[0])
    new_pvalues = empty(n)
    if correction_type == "Bonferroni":
        new_pvalues = n * pvalues
    elif correction_type == "Bonferroni-Holm":
        values = [ (pvalue, i) for i, pvalue in enumerate(pvalues) ]
        values.sort()
        for rank, vals in enumerate(values):
            pvalue, i = vals
            new_pvalues[i] = (n-rank) * pvalue
    elif correction_type == "Benjamini-Hochberg":
        values = [ (pvalue, i) for i, pvalue in enumerate(pvalues) ]
        values.sort()
        values.reverse()
        new_values = []
        for i, vals in enumerate(values):
            rank = n - i
            pvalue, index = vals
            new_values.append((n/rank) * pvalue)
        for i in range(0, int(n)-1):
            if new_values[i] < new_values[i+1]:
                new_values[i+1] = new_values[i]
        for i, vals in enumerate(values):
            pvalue, index = vals
            new_pvalues[index] = new_values[i]
    if needs_reshaping:
        #print('original_shape\n',pvalues.reshape((original_shape)))
        #print('new_pvalues\n',new_pvalues.reshape((original_shape)))
        new_pvalues = new_pvalues.reshape((original_shape))
    #print("new_pvalues\n",new_pvalues)
    return new_pvalues


def do_louvain_merger(in_mat, sample_k_lists, cutoff = 0):
    ## take in the BH corrected p-values for whether 
    ## the groups are significantly different for this ome
    ## at the level of the feature selected affinity matrix
    # actually cut the edges that need to be pruned (apparently setting the weights to 0 doesn't actually give it zero weight...?)
    cut_counter = 0
    for i in range(in_mat.shape[0]):
        for j in range(in_mat.shape[1]):
            if in_mat[i,j] < cutoff:
                in_mat[i,j] = 0
                cut_counter +=1
    print('cut',cut_counter,'low weight edges')
    ## make the fully connected, weighted matrix
    G = nx.from_numpy_matrix(in_mat)
    for node in G.nodes():
        print(node)
    for edge in G.edges():
        print(edge)
    partition = community.best_partition(G)
    ## this is a dictionary with the original group index as the keys
    ## and the new group index as the 
    print(partition)
    ## make the empty lists for holding all of the 
    ## new sample group lists
    new_groups = []
    new_group_annotations = []
    for i in range(len(set(partition.values()))):
        new_groups.append([])
        new_group_annotations.append([])
    for i in range(in_mat.shape[0]):
        new_groups[partition[i]]+=sample_k_lists[i]
        new_group_annotations[partition[i]].append(i)
    return(new_groups,new_group_annotations, partition)













#!/usr/env python3
import os
import sys
import argparse
import numpy as np
import seaborn as sns
import pandas as pd
from sklearn.metrics import mutual_info_score
from sklearn.cluster import AffinityPropagation as ap
#from sklearn.metrics import log_loss
from sklearn.preprocessing import LabelEncoder as label_maker
from sklearn.preprocessing import OneHotEncoder as onehot
from numpy import array
from numpy import argmax
from copy import deepcopy
from sklearn.preprocessing import LabelEncoder
from sklearn.preprocessing import OneHotEncoder
from matplotlib import pyplot as plt
from multiprocessing.dummy import Pool as ThreadPool
from sklearn.metrics import pairwise
from scipy.stats import spearmanr as spear
from scipy.stats import rankdata, fisher_exact, chisquare, mode

from mana_clust.common_functions import write_table, read_table, digitize_for_max_info, get_sample_k_lists, correct_pvalues_for_multiple_testing

euc = pairwise.euclidean_distances

##############################################################################

## hyperparameters
threads = 48

##############################################################################

##############################################################################
def get_sd_from_beta(vect,cutoff=1):
    vect = np.array(list(vect)+list(vect*-1))
    m = np.mean(vect)
    s = np.std(vect)
    return(m+s*cutoff)


def one_hot(data):
    values = np.array(data)
    ## catelogue where all of the Nones are
    none_idxs = np.where(values=="MISSING_DATA")[0]
    cat_idxs = np.where(values!="MISSING_DATA")[0]
    integer_encoded = np.zeros((values.shape[0]))
    # integer encode
    label_encoder = LabelEncoder()
    integer_encoded = label_encoder.fit_transform(values[cat_idxs])
    
    #print(integer_encoded)
    # binary encode
    try:
        onehot_encoder = OneHotEncoder(sparse=False, categories='auto')
    except:
        onehot_encoder = OneHotEncoder(sparse=False)
    integer_encoded = integer_encoded.reshape(len(integer_encoded), 1)
    onehot_encoded = onehot_encoder.fit_transform(integer_encoded)
    ## make the final one hot encoded to account for Nones
    #print(onehot_encoded.shape)
    final_one_hot_encoded = np.zeros((values.shape[0],onehot_encoded.shape[1]))
    for i in range(0,cat_idxs.shape[0]):
        temp_idx = cat_idxs[i]
        final_one_hot_encoded[temp_idx] = onehot_encoded[i]
    #print(onehot_encoded)
    return(integer_encoded, final_one_hot_encoded,label_encoder)
##############################################################################
class feature():

    def __init__(self, data_vect, name):
        self.labels = list(data_vect)
        self.label_array = np.array(self.labels)
        self.label_num, self.label_embedding, self.label_encoder = one_hot(data_vect)
        self.name = name

    def get_name(self):
        return(self.name)

    def set_name(self, new_name):
        self.name = new_name

    def get_label_num(self):
        return(self.label_num)

    def get_embedding(self):
        return(self.label_embedding)

    def get_dim(self):
        return(self.get_embedding().shape)

    def get_col_dim(self):
        return(self.get_dim()[1])

    def inflate_embedding(self,col_dim):
        ## check that it's a valid inflation
        if col_dim < self.get_col_dim():
            err_str = 'cannot inflate the embedding for '+self.get_name()+". Orig dim:"+str(self.get_dim())+'. Tried to set it to:'+str(col_dim)
            sys.exit(err_str)
        new_embedding = np.zeros((len(self.labels),col_dim))
        #print(new_embedding)
        for col in range(self.get_col_dim()):
            new_embedding[:,col] = self.label_embedding[:,col]
        #print(new_embedding)
        self.label_embedding = new_embedding

    def __str__(self):
        print_str = "feature object: "+self.get_name()
        print_str += "\n\t"+str(self.get_embedding())
        return(print_str)


##############################################################################
def remove_invarainat_features(in_mat):
    keep_col_vect = [True]*in_mat.shape[1]
    for i in range(1,in_mat.shape[1]):
        ## after removing missing data how many unique values are there?
        feature_name = in_mat[0,i]
        temp_col_entries = in_mat[1:,i]
        temp_col_entries = temp_col_entries[temp_col_entries != "MISSING_DATA"]
        num_unique_values = len(list(set(temp_col_entries)))
        if num_unique_values == 1 or num_unique_values == temp_col_entries.shape[0]:
            keep_col_vect[i]=False
            print('removing:',feature_name,num_unique_values,temp_col_entries)
    print("keeping",np.sum(keep_col_vect)-1,'features that have some variance')
    rm_vect = np.array(-1* (np.array(keep_col_vect,dtype=int)-1),dtype=bool)
    print('removing:',in_mat[0,rm_vect])
    return(in_mat[:,keep_col_vect])


def filter_none_subjects(in_mat):
    if isinstance(in_mat,list):
        in_mat = np.array(in_mat)
    ## get rid of invariant features
    in_mat = remove_invarainat_features(in_mat)

    ## get rid of subjects with all missing data
    
    out_mat = [in_mat[0,:].tolist()]
    all_none_cutoff = in_mat.shape[1]-1
    rm_subject = []
    for i in range(1,in_mat.shape[0]):
        temp_row = in_mat[i,1:]
        if np.sum(np.array(temp_row=="MISSING_DATA",dtype=int))==all_none_cutoff:
            rm_subject.append(in_mat[i,0])
        else:
            out_mat.append(in_mat[i,:].tolist())
    out_mat = np.array(out_mat)
    print("removed subjects:",rm_subject)
    #out_mat = remove_invarainat_features(out_mat)
    return(out_mat)


def get_randome(randome):
        ## make a randomized version of this categorical ome
        ## shuffle up the labels in each feature
        for i in range(1,np.shape(randome)[1]):
            np.random.shuffle(randome[1:,i])
            #randome[1:,i] = np.random.shuffle(randome[1:,i])
        print(randome)
        randome = categorical_ome(randome)
        return(randome)


def get_contingency_table(group_vector_1, group_vector_2, return_ids = False):
    ## check that they're the same length
    if len(group_vector_1)!=len(group_vector_2):
        sys.exit('get_contingency_table got two vectors of differing lengths')
    ## get the sets, their indices, and make the cont table
    if return_ids:
        grp1_set = sorted(list(set(group_vector_1)))
        grp2_set = sorted(list(set(group_vector_2)))
    else:
        grp1_set = set(group_vector_1)
        grp2_set = set(group_vector_2)
    grp1_set_idx_hash = {value:key for key, value in enumerate(grp1_set)}
    grp2_set_idx_hash = {value:key for key, value in enumerate(grp2_set)}
    ## make the cont table axis0=group1, axis1=group2
    cont_table = np.zeros((len(grp1_set), len(grp2_set)))
    for i in range(len(group_vector_1)):
        row = grp1_set_idx_hash[group_vector_1[i]]
        col = grp2_set_idx_hash[group_vector_2[i]]
        cont_table[row,col]+=1
    if not return_ids:
        return(cont_table)
    if return_ids:
        return(cont_table, grp1_set, grp2_set)



def log_loss(y_true, y_pred, eps = 1e-15):
    if not isinstance(y_true, np.ndarray) or not isinstance(y_pred, np.ndarray):
        print("\n\n",y_true,"\n",y_pred,"\n\n")
        print("didn't get arrays for y_true and y_pred")
        return(None)
    if np.shape(y_true)!=np.shape(y_pred):
        sys.exit('y_true and y_pred are not equal shape')
    ## now do the calculation for the log loss
    y_pred = np.clip(y_pred, eps, 1 - eps)
    y_true = np.clip(y_true, eps, 1 - eps)
    ll = np.sum(-y_true*np.log(y_pred) - (1-y_true)*np.log(1-y_pred))
    ## check for a weird result
    if np.isnan(ll) or np.isinf(ll) or np.isneginf(ll):
        print('got a weirdo')
        print(ll)
        sys.exit()
    return(ll)


class categorical_ome():
    """ this class is for any categorical ome """
    def __init__(self, in_mat, path = None, name = None, plot = False):
        ## rows are subjects, columns are features
        if name != None:
            self.name = name
        elif path != None:
            self.name = os.path.splitext(path)[0].split('/')[-1]
            print('\t',self.name)
        else:
            self.name = None
        self.original_mat = filter_none_subjects(in_mat)
        self.randomized_original_mat = deepcopy(self.original_mat)
        self.feature_names = self.original_mat[0,1:]
        self.all_feature_names = deepcopy(self.feature_names)
        self.num_features = self.feature_names.shape[0]
        self.full_cat_data = self.original_mat[1:,1:]
        self.subjects = self.original_mat[1:,0]
        self.num_subjects = np.shape(self.subjects)[0]
        self.plot = plot
        self.all_diff = None
        self.real_mi_mat = None
        self.min_diff = None
        self.max_of_min_diff = None

        self.all_features = []
        for index, temp_feature in enumerate(self.feature_names):
            #print(one_hot(full_cat_data[:,index]))
            self.all_features.append(feature(self.full_cat_data[:,index],temp_feature))

    def get_noise_factor(self,division_factor=1):
        non_zero = self.all_log_loss>1e-10
        #sns.distplot(self.all_log_loss[non_zero])
        #plt.show()
        #plt.clf()
        return(np.min(self.all_log_loss[non_zero]/division_factor))
        #return(np.mean(self.all_log_loss)/division_factor)

    def smooth_log_loss(self,smooth_iters = 1):
        noise_factor = self.get_noise_factor()
        temp_log_loss = self.all_log_loss[:]
        for i in range(smooth_iters):
            temp_log_loss+=np.random.normal(loc=0.0,scale=noise_factor,size=self.all_log_loss.shape)
        self.all_log_loss = temp_log_loss

    def process_cat_ome(self):
        ## go through each feature, finding the highest col_dim
        self.all_col_dim = []
        for f in self.all_features:
            self.all_col_dim.append(f.get_col_dim())


        ## go through each feature, inflating the one hot encoding matrices to enable concatenation
        self.max_col_dim = max(self.all_col_dim)
        print('max embedding dim:',self.max_col_dim)
        for f in self.all_features:
            self.all_col_dim.append(f.inflate_embedding(self.max_col_dim))

        ## make the encoded matrix
        ## dimtions are: [feature,subject,class within feature]
        self.all_encodings = []
        for f in self.all_features:
            #print(f.get_embedding())
            self.all_encodings.append(f.get_embedding())
        self.all_encodings = np.array(self.all_encodings)

        ## get all of the log_loss distances
        self.all_log_loss = np.zeros((self.num_subjects,self.num_subjects))
        ## generate the index pairs for doing the mapping
        index_pairs = []
        for i in range(0,self.num_subjects):
            for j in range(i,self.num_subjects):
                if i!=j:
                    index_pairs.append([i,j])

        #################################################
        ## parallelize the log losses
        print("getting log losses")

        pool = ThreadPool(threads)
        ll_pairs = pool.map(self.get_log_loss,index_pairs)
        pool.close()
        pool.join()

        #print(ll_pairs)
        #print(list(self.all_log_loss))
        print("finished getting log losses. Generating affinity matrix")
        for k in range(0,len(index_pairs)):
            i = index_pairs[k][0]
            j = index_pairs[k][1]
            self.all_log_loss[i,j] = ll_pairs[k]
            self.all_log_loss[j,i] = ll_pairs[k]


        self.smooth_log_loss()

        print(self.all_log_loss)

        ## adjust log loss to accomodate subjects that had no overlapping variables
        print('inf')
        print(np.where(np.isinf(self.all_log_loss)))
        print('neg inf')
        print(np.where(np.isneginf(self.all_log_loss)))
        print('nan')
        print(np.where(np.isnan(self.all_log_loss)))
        none_row, none_col = np.where(self.all_log_loss==None)
        nan_row, nan_col = np.where(np.isnan(self.all_log_loss))
        # for i in range(neg_inf_row.shape[0]):
        #   print(self.get_log_loss([neg_inf_row[i], neg_inf_col[i]], verbose=True))

        mean_log_loss = np.nanmean(self.all_log_loss)
        print(self.all_log_loss[nan_row, nan_col])
        # if nan_row.shape[0] > 0:
        #   for i in range(nan_row.shape[0]):
        #       self.all_log_loss[nan_row[i],nan_col[i]] = mean_log_loss
        # if none_row.shape[0] > 0:
        #   for i in range(none_row.shape[0]):
        #       self.all_log_loss[none_row[i],none_col[i]] = mean_log_loss
        self.all_log_loss[nan_row, nan_col] = mean_log_loss
        self.all_log_loss[none_row, none_col] = mean_log_loss
        print(self.all_log_loss[nan_row, nan_col])

        print('should all be fixed now:')
        print(np.where(np.isinf(self.all_log_loss)))
        print(np.where(np.isneginf(self.all_log_loss)))
        print(np.where(np.isnan(self.all_log_loss)))
        # self.all_spear,dummy = spear(self.all_log_loss)
        # del dummy

        # self.affinity_matrix = -euc(self.all_spear,squared=True)
        self.affinity_matrix = -euc(self.all_log_loss, squared=True)
        # self.affinity_matrix = -1*np.reshape(rankdata(self.affinity_matrix.flatten()),self.affinity_matrix.shape)
        # print(self.affinity_matrix)
        # print(self.affinity_matrix.shape)
        non_diag_indices = ~np.eye(self.affinity_matrix.shape[0],dtype=bool)
        non_diag_max = np.max(self.affinity_matrix[non_diag_indices])
        ## subtracting the negative, so it brings the max up to zero
        self.affinity_matrix[non_diag_indices] -= non_diag_max
        self.affinity_matrix = self.affinity_matrix/(np.min(self.affinity_matrix)/-100)
        print(self.affinity_matrix)
        print(np.min(self.affinity_matrix))

    def filter_out_nones(self,temp_a,temp_b):
        """ this takes in subject a and b & filteres out any features that have missing values """
        #print('looking at a/b')
        #print(temp_a)
        #print(temp_b)
        ## sum across the feature axis to figure out which features are missing a class
        temp_a_sum = np.sum(temp_a,axis=1)
        temp_b_sum = np.sum(temp_b,axis=1)
        ## if either are zero, then don't consider them
        either_non = temp_a_sum * temp_b_sum
        num_features = np.sum(either_non)
        #print(temp_a_sum)
        #print(temp_b_sum)
        #print(either_non)
        #print(num_features)
        ## sum them from both individuals

        mask = np.array(either_non,dtype=bool)
        #print(mask)
        #print(temp_a)
        #print(temp_b)
        masked_temp_a = temp_a[mask,:]
        masked_temp_b = temp_b[mask,:]
        #print(temp_a)
        #print(temp_b)
        # if not isinstance(masked_temp_a,np.ndarray) or not isinstance(masked_temp_b,np.ndarray):
        #   print("found the place where there are non-arrays")
        # print('\n')
        # print(temp_a)
        # print(temp_b)
        # print(masked_temp_a, masked_temp_b)
        # print(num_features)
        return(masked_temp_a, masked_temp_b,num_features)

    def get_log_loss(self, indices,verbose=False):
        #global all_encodings#, all_log_loss
        i, j = indices
        #print(all_encodings[:,i,:])
        #print(all_encodings[:,j,:])
        temp_a = self.all_encodings[:,i,:]
        temp_b = self.all_encodings[:,j,:]

        temp_a_masked, temp_b_masked, num_features = self.filter_out_nones(temp_a,temp_b)
        if num_features==0:
            return(None)
        else:
            ll = log_loss(temp_a_masked,temp_b_masked)/num_features
            return(ll)
        # if verbose:
        #   print(temp_a)
        #   print(temp_b)
        #   print(num_features)
        #   print(temp_a_masked)
        #   print(temp_b_masked)
        #   print(ll)
        #all_log_loss[i,j] = ll
        #all_log_loss[j,i] = ll
    
    def plot_all_and_good_features(self, do_cluster = True, out_dir = None):
        sns.clustermap(self.real_mi_mat,row_cluster = do_cluster, col_cluster = do_cluster)
        plt.show()
        plt.clf()

        sns.clustermap(self.real_mi_mat - self.all_diff[0,:,:], row_cluster = do_cluster, col_cluster = do_cluster)
        plt.show()
        plt.clf()

        sns.clustermap(self.all_diff[0,:,:],row_cluster = do_cluster, col_cluster = do_cluster)
        plt.show()
        plt.clf()

        sns.clustermap(self.min_diff)
        plt.show()
        plt.clf()

        sns.distplot(self.max_of_min_diff)
        plt.show()
        plt.clf()




    def get_good_features(self, rand_iters = 10, percent_better_than_random=0.9, do_plot = False):
        ## first calculate the observed mutual information across all features
        print('getting information for real features')
        self.real_mi_mat = self.get_cross_feature_info()
        temp_original_mat = self.randomized_original_mat[:]
        print('actual original:\n',temp_original_mat)
        #sys.exit()
        ## make rand_iters number of random datasets, catelogueing the mutual
        ## information across each of the features
        self.all_diff = np.zeros((rand_iters,self.num_features,self.num_features))
        for i in range(rand_iters):
            print('\tgetting rand-ome information: iter',i)
            temp_randome = get_randome(self.randomized_original_mat[:])
            temp_mi = temp_randome.get_cross_feature_info()
            self.all_diff[i,:,:] = self.real_mi_mat - temp_mi

        print('actual original:\n',self.original_mat)
        print('new original:\n',self.original_mat)
        print(temp_original_mat == self.original_mat)
        
        ## now get the features that look good!
        num_better_than_random = int(percent_better_than_random * rand_iters)
        self.min_diff = np.min(self.all_diff,axis=0)
        self.max_of_min_diff = np.max(self.min_diff,axis=0)
        self.all_diff_bool = self.all_diff > 0
        self.num_times_pair_better_than_random = np.sum(self.all_diff_bool, axis=0)
        if do_plot:
            self.plot_all_and_good_features()
            
        self.good_feature_indices = np.where(self.max_of_min_diff>0.15)[0]#get_sd_from_beta(self.max_of_min_diff))[0]
        print('\n\nfound',self.good_feature_indices.shape[0],'good features out of',self.max_of_min_diff.shape[0])
        return(categorical_ome(self.original_mat[:,np.array([0]+list(self.good_feature_indices+1),dtype=int)], name = self.name+"_high_info"))
        # sns.clustermap(self.num_times_pair_better_than_random)
        # plt.show()
        # plt.clf()

        #per_feature_avg_signficance = np.mean(self.num_times_pair_better_than_random,percent_better_than_random)


    def get_cross_feature_info(self):
        ## 
        print("\n\ngetting cross feature information:",len(self.all_features),"features")
        feature_index_pairs = []
        for i in range(0,len(self.all_features)):
            for j in range(i,len(self.all_features)):
                if i != j:
                    feature_index_pairs.append([i,j])
        print('\n\n\t\t',len(feature_index_pairs),"comparisons")
        pool = ThreadPool(threads)
        mi_pairs = pool.map(self.cross_feature_mutual_info,feature_index_pairs)
        pool.close()
        pool.join()

        ## log all of the mutual information across pairs of features
        pairwise_mi = np.zeros((self.num_features,self.num_features))
        for k in range(0,len(feature_index_pairs)):
            i = feature_index_pairs[k][0]
            j = feature_index_pairs[k][1]
            pairwise_mi[i,j] = mi_pairs[k]
            pairwise_mi[j,i] = mi_pairs[k]
        return(pairwise_mi)

    def cross_feature_mutual_info(self, indices):
        """Takes in two features & make a cont table & get the mutual information"""
        feat1_idx, feat2_idx = indices
        feat1 = self.all_features[feat1_idx]
        feat2 = self.all_features[feat2_idx]

        if feat1.label_array.shape != feat2.label_array.shape:
            sys.exit("feature dimentions don't match when trying to make cont table")
        else:
            f1_not_missing = np.array(feat1.label_array!="MISSING_DATA", dtype=int)
            f2_not_missing = np.array(feat2.label_array!="MISSING_DATA", dtype=int)
            present_in_both = np.array(f1_not_missing * f2_not_missing, dtype=bool)
            if np.sum(present_in_both)==0:
                ## if there is no overlap in feature presence, then return 0,
                ## because there is no information
                return(0)
            f1_labels = feat1.label_array[present_in_both]
            f2_labels = feat2.label_array[present_in_both]

            # df = pd.DataFrame({"f1":f1_labels,
            #                  "f2":f2_labels})
            # ## get the cross tab frequencies
            # freq = pd.crosstab(index=df["f1"], 
            #                    columns=df["f2"])
            freq = get_contingency_table(f1_labels, f2_labels)
            mi = mutual_info_score(None, None, contingency = freq)
            #print(freq, mi)
        return(mi)


    ############################
    def get_feature_by_group_contingency_table(self, index, labels = None):
        ## if there aren't labels given, assume it's a fully crossed all groups, all labels cont-table
        if labels == None:
            labels = self.sample_k_list_labels

        temp_vect = self.full_cat_data[:,index]
        all_entries = sorted(list(set(temp_vect)))
        #print(all_entries)

        if "MISSING_DATA" in all_entries:
            all_entries_array = np.array(all_entries)
            all_entries = all_entries_array[all_entries_array!="MISSING_DATA"].tolist()
            #print(all_entries)
        else:
            pass

        all_entry_position_dict = {value:key for key, value in enumerate(all_entries)}
        cont_mat = np.zeros((len(labels), len(all_entries)))

        for i in range(len(labels)):
            for j in range(len(labels[i])):
                temp_entry = temp_vect[labels[i][j]]
                #print(temp_entry)
                if temp_entry != "MISSING_DATA":
                    col_idx = all_entry_position_dict[temp_entry]
                    cont_mat[i,col_idx] += 1

        return(cont_mat)


    def single_fisher(self, index):
        #print(index)
        cont_mat = self.get_feature_by_group_contingency_table(index)
        #print(cont_mat)
        stat, p = chisquare(cont_mat, axis = None)
        return([stat, p])


    def get_two_group_chi(self, first, second):
        all_groups = list(set(first+second))
        group_indexes = {value:key for key, value in enumerate(all_groups)}
        cont_table = np.zeros((2,len(all_groups)))
        for i in range(len(first)):
            cont_table[0,group_indexes[first[i]]] += 1
        for i in range(len(second)):
            cont_table[1,group_indexes[second[i]]] += 1
        chi, p = chisquare(cont_table, axis = None)
        return(chi, p)


    def do_pairwise(self, index, prepend_str = "sample_group_"):
        temp_name = self.feature_names[index]
        if False:#'IgE' in temp_name:
            verbose = True
        else:
            verbose = False
        temp_data = self.full_cat_data[:,index]
        temp_results = []
        #print(temp_name)
        for i in range(0,len(self.sample_k_list_labels)):
            if len(self.sample_k_list_labels[i])>=5:
                if verbose:
                    print('\t',i,'had more than 5')
                for j in range(i,len(self.sample_k_list_labels)):
                    if i!=j:
                        if len(self.sample_k_list_labels[j])>=5:
                            if verbose:
                                print('\t',j,'had more than 5')
                            first = temp_data[self.sample_k_list_labels[i]]
                            first = first[np.where(first!="MISSING_DATA")[0]].tolist()
                            second = temp_data[self.sample_k_list_labels[j]]
                            second = second[np.where(second!="MISSING_DATA")[0]].tolist()
                            if len(first)>=5 and len(second)>=5:
                                mode_1 = mode(first)[0][0]
                                mode_2 = mode(second)[0][0]
                                if verbose:
                                    print('\t\tboth more than 5')
                                    print('\t\tmode1:',mode_1)
                                    print('\t\tmode2:',mode_2)
                                if mode_1 != mode_2:
                                    temp_chi, temp_p = self.get_two_group_chi(first, second)
                                    temp_results.append([temp_name,
                                                         prepend_str+str(i),
                                                         prepend_str+str(j),
                                                         mode_1,
                                                         mode_2,
                                                         temp_chi,
                                                         temp_p])
                                    if verbose:
                                        print("\t\t\t",temp_results[-1])
        return(temp_results)


    def summarize_pairwise_group_differences(self, final_output, out_dir, prepend_str = "sample_group_"):
        num_diff_mat = np.zeros((len(self.sample_k_list_labels),len(self.sample_k_list_labels)))
        group_names = []
        for i in range(len(self.sample_k_list_labels)):
            group_names.append(prepend_str+str(i))
        group_index_dict = {value:key for key, value in enumerate(group_names)}
        for line in final_output:
            grp1_idx = group_index_dict[line[1]]
            grp2_idx = group_index_dict[line[2]]
            num_diff_mat[grp1_idx, grp2_idx] += 1
            num_diff_mat[grp2_idx, grp1_idx] += 1
        sns.clustermap(num_diff_mat)
        plt.savefig(out_dir+self.name+'_number_different_across_groups.png',
                dpi=600,
                bbox_inches='tight')
        plt.clf()


    def do_stats(self, labels, out_dir, alpha=0.05, processes = 1, prepend_str = "sample_group_"):
        idx_list = list(range(self.full_cat_data.shape[1]))
        self.sample_k_list_labels = get_sample_k_lists(labels)
        arg_list = []
        for i in idx_list:
            arg_list.append(i)
        pool = ThreadPool(threads)
        global_fishers = pool.map(self.single_fisher,arg_list)
        pool.close()
        pool.join()
        # global_fishers = []
        # for i in arg_list:
        #     global_fishers.append(self.single_fisher(i))
        #     print(global_fishers[-1])
        global_fishers_array = np.array(global_fishers)
        self.statistic = global_fishers_array[:,0]
        self.uncorrected_p = global_fishers_array[:,1]
        self.corrected_p = correct_pvalues_for_multiple_testing(self.uncorrected_p.tolist())
        self.global_sig_indices = list(np.where(self.corrected_p < alpha)[0])
        self.global_sig_indices += list(np.where(self.corrected_p == 0. )[0])## for whatever reason, zeros weren't coming out...
        self.global_sig_indices = np.array(sorted(list(set(self.global_sig_indices))),dtype = int)
        for i in range(len(self.feature_names)):
            print(self.feature_names[i],self.corrected_p[i], i in self.global_sig_indices)
        print('\tfound',self.global_sig_indices.shape[0] , 'significant differences')

        ## write the results as a table
        feature_names = np.concatenate((np.array(['features']),self.feature_names))
        statistic = np.concatenate((np.array(['chi_statistic']),self.statistic))
        uncorrected_p  = np.concatenate((np.array(['uncorrected_p_value']),self.uncorrected_p))
        corrected_p  = np.concatenate((np.array(['BH_corrected_p_value']),self.corrected_p))
        global_out_table = np.transpose(np.concatenate((feature_names[None,:],statistic[None,:], uncorrected_p[None,:], corrected_p[None,:]), axis = 0))
        write_table(global_out_table,out_dir+self.name+"_global_statistical_differences.tsv")

        print("\tstarting to do protected pairwise analyses")

        pool = ThreadPool(threads)
        pairwise_results = pool.map(self.do_pairwise,self.global_sig_indices)
        pool.close()
        pool.join()

        while len(pairwise_results)>1:
            pairwise_results[0] = pairwise_results[0]+pairwise_results[1]
            del pairwise_results[1]
        if len(pairwise_results) > 0:
            pairwise_results = np.array(pairwise_results[0])
        if pairwise_results.shape[0] > 0:
            temp_uncorrected_p = np.array(pairwise_results[:,-1], dtype = float).tolist()
            temp_corrected_p = correct_pvalues_for_multiple_testing(temp_uncorrected_p)
            ## filter out all of the results that weren't significant
            final_output = []
            for i in range(0,len(temp_corrected_p)):
                if temp_corrected_p[i] < alpha:
                    temp_line = pairwise_results[i,:].tolist()+[temp_corrected_p[i]]
                    final_output.append(temp_line)

            print('\ttotal of',len(final_output),'signficant post-hocs')    
            ## summarize the number of differences between each group
            self.summarize_pairwise_group_differences(final_output, out_dir)

            ## write the summary table
            header = np.array(([['feature_name','group_1','group_2','mode_1','mode_2','chi_statistic','uncorrected_p','BH_corrected_p']]))
            final_output = np.concatenate((header, final_output))
        else:
            header = np.array(([['feature_name','group_1','group_2','mode_1','mode_2','chi_statistic','uncorrected_p','BH_corrected_p']]))
            final_output = np.array(([['None','None','None','None','None','None','None','None']]))
            final_output = np.concatenate((header, final_output))
        write_table(final_output,out_dir+self.name+"_all_significant_postHocs.tsv")

        return

    ############################

    def plot_heatmaps(self, do_cluster=True, out_dir = None):
        ## plot the log loss


        sns.clustermap(self.all_log_loss,row_cluster = do_cluster, col_cluster = do_cluster)
        plt.show()
        plt.clf()


        # sns.clustermap(self.real_mi_mat,row_cluster = do_cluster, col_cluster = do_cluster)
        # plt.show()
        # plt.clf()

        # sns.clustermap(self.real_mi_mat - self.all_diff[0,:,:],row_cluster = do_cluster, col_cluster = do_cluster)
        # plt.show()
        # plt.clf()

        # sns.clustermap(self.all_diff[0,:,:],row_cluster = do_cluster, col_cluster = do_cluster)
        # plt.show()
        # plt.clf()

        # sns.clustermap(self.min_diff)
        # plt.show()
        # plt.clf()

        # sns.distplot(self.max_of_min_diff)
        # plt.show()
        # plt.clf()

        # sns.clustermap(self.all_log_loss,row_cluster = do_cluster, col_cluster = do_cluster)
        # plt.show()
        # plt.clf()
        

        return


##############################################################################

##############################################################################
def parse_args(args):
    ##########################################################################
    parser = argparse.ArgumentParser()

    ## global arguments
    parser.add_argument(
        '-infile','-in','-i','-input',
        dest='infile',
        nargs='+',
        type=str)

    parser.add_argument(
            '-missing_data','-missing','-md','-ms',
            dest='missing_str',
            help = "The string that should be interpreted as missing data. (Case sensitive)",
            default = "NA",
            type=str)

    parser.add_argument(
            '-do_clust',
            dest='clust',
            help='if you want to do ap clustering here directly',
            action='store_true')

    parser.add_argument(
            '-plot',
            dest='plot',
            help='if you want to make some plots',
            action='store_true')

    args = parser.parse_args()
    return(args)
##############################################################################
## process the input file
def get_in_mat(infile,missing_str):
    in_mat = read_table(infile)
    in_mat = digitize_for_max_info(in_mat)
    for index, row in enumerate(in_mat):
        for idx2, element in enumerate(row):
            if row[idx2] == missing_str:
                row[idx2] = "MISSING_DATA"
        in_mat[index] = row
    return(np.array(in_mat))


def do_clust(cat_ome):
    am = cat_ome.affinity_matrix
    af = ap(preference = np.min(am,axis=1),affinity="precomputed").fit(am)
    labels = af.labels_
    for idx, subject in enumerate(cat_ome.subjects):
        print(cat_ome.subjects[idx]+'\t'+str(labels[idx]))



##############################################################################
## be able to do it for more than one cat ome
def do_cat_ome(infile,missing_str, do_cluster = True, plot = False, clust = False):
    in_mat = get_in_mat(infile, missing_str)


    ## test with low none input
    cat_ome = categorical_ome(in_mat, path = infile)
    important_cat_ome = cat_ome.get_good_features(do_plot = plot)

    print("high info mat:",important_cat_ome.original_mat.shape)
    print(important_cat_ome.original_mat)
    cat_ome.process_cat_ome()
    important_cat_ome.process_cat_ome()


    if plot:
        important_cat_ome.plot_heatmaps()


    # if plot:
    #     print('plotting original')
    #     plot_stuff(cat_ome)

    #     print('plotting high info')
    #     plot_stuff(important_cat_ome)

    if clust:
        print("\n\n\n\ndoing clustering on full mat")
        do_clust(cat_ome)
        print("\n\n\n\ndoing clustering on important mat")
        do_clust(important_cat_ome)

    return(cat_ome, important_cat_ome)

def do_all_cat_omes(infile_list, missing_str, plot = False):
    all_full_cat_omes = []
    all_cat_omes = []
    for infile in infile_list:
        temp_full_cat_ome, temp_important_cat_ome = do_cat_ome(infile, missing_str, plot = plot)
        all_full_cat_omes.append(temp_full_cat_ome)
        all_cat_omes.append(temp_important_cat_ome)
    return(all_full_cat_omes, all_cat_omes)


##############################################################################

if __name__ == "__main__":
    ## set up the parser
    args = parse_args(sys.argv)
    all_full_cat_omes, all_cat_omes = do_all_cat_omes(args.infile, args.missing_str, plot = args.plot)
    # if args.plot:
    #     for cat in all_cat_omes:
    #         cat.plot_heatmaps()





import argparse
import random
import seaborn as sns
from matplotlib import pyplot as plt
from copy import deepcopy
import numpy as np
import pandas as pd
from scipy.stats import ttest_rel
from sklearn.metrics import mutual_info_score

from mana_clust.simulate_datasets import make_all_datasets
from mana_clust.mana_clust import cluster_omes, write_ome_results, plot_ome_results
from mana_clust.simulate_datasets import process_dir
from mana_clust.common_functions import write_table
from mana_clust.mana_cat import get_contingency_table




###########################################################################
def parse_args():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-out_dir',
        default = "/home/scott/bin/manaclust/lib/synthetic_data/",
        type=str)

    parser.add_argument(
        '-iters','-iterations',
        default = 2,
        type=int)

    parser.add_argument(
        '-num_cat_groups',
        default = 2,
        type=int)

    parser.add_argument(
        '-num_cat_datasets','-cat_dat',
        default = 1,
        type=int)

    parser.add_argument(
        '-cat_missing','-cat_missing',
        default = .50,
        type=float)

    parser.add_argument(
        '-cat_noise',
        default = .50,
        type=float)

    parser.add_argument(
        '-num_cat_rand_feat',
        default = 150,
        type=int)

    parser.add_argument(
        '-num_cat_real_feat',
        default = 15,
        type=int)

    parser.add_argument(
        '-num_num_groups',
        default = 3,
        type=int)

    parser.add_argument(
        '-num_num_datasets','-num_dat',
        default = 2,
        type=int)

    parser.add_argument(
        '-samples','-s',
        default = 1000,
        type=int)

    parser.add_argument(
        '-seed',
        default = 123456789,
        type=int)

    args = parser.parse_args()
    return(args)
###########################################################################    

def do_mi_line_plot(mi_df, paired_t_test_t, paired_t_test_p, out_dir, y="mutual information", name="clustering_results", size = 5, y_min = None, y_max = None):
    ax = sns.pointplot(x="method", y=y, hue = "iter",
                       data = mi_df, 
                       size=size, 
                       scale = 3,
                       legend = False,
                       alpha = 0.5,
                       dodge = True)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    ax.get_legend().remove()
    out_dir = process_dir(out_dir)
    out_file = out_dir+name+"_"+y.replace(' ','_')+"_lines.png"
    sns.set(font_scale=1.75)

    if y_min != None and y_max != None:
        plt.ylim(y_min,y_max)
    
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf()
    return



def get_purity(contingency_matrix, true_axis = 1):
    ## takes in a contingency table where the ground truth is in the rows, and the 
    ## clustering results are in the columns and returns the purity
    purity = np.sum(np.amax(contingency_matrix, axis = true_axis)) / np.sum(contingency_matrix)
    return(purity)


def get_real_random_feat_stats(selected_features, all_features, iteration, method = "MANAclust"):
    ## make the contingency table of real vs random features

    ## Make dicts for the selected and non-selected features
    selected_dict = {feature:index for index, feature in enumerate(selected_features)}
    non_selected = []
    non_selected_dict = {}
    real_features = []
    random_features = []
    for feature in all_features:
        try:
            selected_dict[feature]
        except:
            non_selected.append(feature)
            non_selected_dict[feature]=len(non_selected)-1
        if feature[-4:] == 'real':
            real_features.append(feature)
        else:
            random_features.append(feature)
    ## Go through each of the features making the contingency table 
    cont_table = np.zeros((2,2))
    ##################################################
    ################   | Actual = Yes  |  Actual = No
    ## predicted = Yes |     X         |     X        
    ## predicted = No  |     X         |     X

    for feat in real_features:
        if feat in selected_dict:
            cont_table[0,0]+=1
        else:
            cont_table[1,0]+=1
    for feat in random_features:
        if feat in selected_dict:
            cont_table[0,1]+=1
        else:
            cont_table[1,1]+=1
    
    TP = cont_table[0,0]
    FP = cont_table[0,1]
    FN = cont_table[1,0]
    TN = cont_table[1,1]
    accuracy = (TP + TN) / np.sum(cont_table)
    recall = TP / (TP + FN)
    precision = TP / (TP + FP)
    specificity = TN / (TN + FP)
    F_one = (2 * precision * recall) / (precision + recall)
    if np.isnan(F_one):
        F_one = 0
    print('\tfeature selection stats:')
    print('\t\tTP:',TP)
    print('\t\tFP:',FP)
    print('\t\tFN:',FN)
    print('\t\tTN:',TN)
    print('\t\taccuracy:',accuracy)
    print('\t\trecall:',recall)
    print('\t\tprecision:',precision)
    print('\t\tspecificity:',specificity)
    print('\t\tF1:',F_one)
    temp_result_dict = {"method":[method],
                        "iter":[iteration],
                        "TP":[TP],
                        "FP":[FP],
                        "FN":[FN],
                        "TN":[TN],
                        "accuracy":[accuracy],
                        "recall":[recall],
                        "precision":[precision],
                        "specificity":[specificity],
                        "F1":[F_one]}
    temp_result_df = pd.DataFrame(temp_result_dict)
    return(temp_result_df)

def get_accuracy_of_feat_select(meta_ome, iteration):
    for i in range(len(meta_ome.cat_omes)):
        select_cat_names = meta_ome.cat_omes[i].feature_names
        full_cat_names = meta_ome.cat_omes_full[i].feature_names
        #print(select_cat_names)
        #print(full_cat_names)
        ## now run the same thing, but with a random selection of features
        num_select = len(select_cat_names)
        random_select_feats = np.random.choice(full_cat_names, size = num_select, replace = False)
        random_feat_select = get_real_random_feat_stats(random_select_feats, full_cat_names, iteration, method = "random")
        real_cat_feat_select = get_real_random_feat_stats(select_cat_names, full_cat_names, iteration)
        cat_feat_select = random_feat_select.append(real_cat_feat_select, ignore_index=True)
    for ome in meta_ome.num_omes:
        select_num_names = ome.feature_names[ome.anti_cor_indices]
        full_num_names = ome.feature_names
        #print(select_num_names)
        #print(full_num_names)
        ## now run the same thing, but with a random selection of features
        num_select = len(select_num_names)
        random_select_feats = np.random.choice(full_num_names, size = num_select, replace = False)
        random_feat_select = get_real_random_feat_stats(random_select_feats, full_num_names, iteration, method = "random")
        real_num_feat_select = get_real_random_feat_stats(select_num_names, full_num_names, iteration)
        num_feat_select = random_feat_select.append(real_num_feat_select, ignore_index=True)
    return(cat_feat_select, num_feat_select)

def get_performance_stats(ground_truth, meta_ome, out_dir):
    out_dir = process_dir(out_dir)
    ## get contingency table from feature selected
    feat_select_cont, feat_select_row, feat_select_col = get_contingency_table_stats(ground_truth, meta_ome.labels, out_dir+"contingency_table_feature_selected.png")
    ## and full
    full_cont, full_row, full_col = get_contingency_table_stats(ground_truth, meta_ome.labels_full, out_dir+"contingency_table_full.png")

    ## get the mutual information
    feat_select_mi = mutual_info_score(None, None, contingency = feat_select_cont)
    full_mi = mutual_info_score(None, None, contingency = full_cont)

    ## get the purity
    feat_select_purity = get_purity(feat_select_cont)
    full_purity = get_purity(full_cont)
    return(feat_select_mi, full_mi, feat_select_purity, full_purity)


def re_order_cont_df(in_df):
    pass
    # already_used = []
    # for i in range(0,in_df.shape[0]):
    #     np.argmax(in_df, axis = 1)


def get_contingency_table_stats(ground_truth, cluster_vect, out_file):
    # cont_table = get_contingency_table(ground_truth, cluster_vect, return_ids = False)
    # print(cont_table)
    cont_table, row_ids, col_ids = get_contingency_table(ground_truth, cluster_vect, return_ids = True)
    row_sums = np.sum(cont_table,axis=1)
    col_sums = np.sum(cont_table,axis=0)
    row_order = np.argsort(row_sums)
    col_order = np.argsort(col_sums)
    col_names_reordered = np.array(col_ids)[col_order]
    row_names_reordered = np.array(row_ids)[row_order]
    reordered_cont_table = cont_table[row_order,:]
    reordered_cont_table= reordered_cont_table[:,col_order]
    cont_df = pd.DataFrame(reordered_cont_table)
    cont_df.columns = col_names_reordered
    index_dict={key:value for key,value in enumerate(row_names_reordered)}
    cont_df.rename(index=index_dict, inplace=True)
    sns.heatmap(cont_df)
    ## re-order the columns to be on the diagonal
    plt.savefig(out_file,
                dpi=600,
                bbox_inches='tight')
    plt.clf()
    return(cont_table, row_ids, col_ids)


def get_group_from_clust_ome(meta_ome):
    all_subjects = meta_ome.all_subjects
    all_groups = []
    for subj in all_subjects:
        all_groups.append(subj.split('|')[-1])
    return(all_groups)


def get_mi_stats(all_feat_mi, 
                 all_full_mi, 
                 feat_select_purity, 
                 full_purity, 
                 out_dir):
    size = 7
    all_feat_mi = list(all_feat_mi)
    all_full_mi = list(all_full_mi)
    paired_t_test_t, paired_t_test_p = ttest_rel(all_feat_mi, all_full_mi)
    print("mutual information paired t-test:",paired_t_test_t, paired_t_test_p)
    method = ["full dataset"]*len(all_full_mi)+["feature selection"]*len(all_feat_mi)
    iteration_vect = np.arange(len(all_full_mi)).tolist()
    iteration_vect = iteration_vect + iteration_vect
    mutual_information = all_full_mi + all_feat_mi
    purity = full_purity + feat_select_purity
    mi_df = pd.DataFrame({"iter":iteration_vect,
                          "method":method,
                          "mutual information":mutual_information,
                          "purity":purity})
    
    ## plot mi violin plot
    out_dir = process_dir(out_dir)
    out_file = out_dir+"clustering_results_mutual_information.png"
    
    ax = sns.violinplot(x="method", y="mutual information", data = mi_df,
                        scale = 'width', 
                        size = size)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf()

    ax = sns.violinplot(x="method", y="mutual information", data = mi_df,
                        scale = 'width', 
                        size = size)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf()

    sns.violinplot(x="method", y="mutual information", data = mi_df,
                        scale = 'width', 
                        size = size)
    ax = sns.swarmplot(x="method", y="mutual information",# hue="method",
                  data=mi_df, palette="muted",
                  edgecolor = 'white', linewidth=1, s=22)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf()
    
    ## do the mi line plot
    do_mi_line_plot(mi_df, paired_t_test_t, paired_t_test_p, out_dir, size = size)
    plt.clf()
    do_mi_line_plot(mi_df, paired_t_test_t, paired_t_test_p, out_dir, size = size)
    plt.clf()

    #### plot purity
    paired_t_test_t, paired_t_test_p = ttest_rel(feat_select_purity, full_purity)
    print("purity paired t-test:",paired_t_test_t, paired_t_test_p)
    ## make the plot
    sns.violinplot(x="method", y="purity", data = mi_df, 
                        scale = "width",
                        size = size)
    ax = sns.swarmplot(x="method", y="purity",# hue="method",
                  data=mi_df, palette="muted",
                  edgecolor = 'white', linewidth=1, s=22)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    out_dir = process_dir(out_dir)
    out_file = out_dir+"clustering_results_purity.png"
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf()
    do_mi_line_plot(mi_df, paired_t_test_t, paired_t_test_p, out_dir, y="purity", size = size)  
    plt.clf()

    ## replot mi violin... 
    paired_t_test_t, paired_t_test_p = ttest_rel(all_feat_mi, all_full_mi)
    out_file = out_dir+"clustering_results_mutual_information.png"
    sns.violinplot(x="method", y="mutual information", data = mi_df, 
                        scale = "width",
                        size = size)
    ax = sns.swarmplot(x="method", y="mutual information",# hue="method",
                  data=mi_df, palette="muted",
                  edgecolor = 'white', linewidth=1, s=22)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf() 
    mi_df.to_csv(out_dir+'clustering_results_summary.tsv',sep='\t')
    return(mi_df)


def do_feat_select_plot(in_stats, var, out_dir, feature_type):
    ## get the methods vectors
    unique_methods = set(in_stats["method"])
    rand_row = in_stats["method"]=="random"
    mom_row = in_stats["method"]=="MANAclust"
    random_results = in_stats[var][rand_row]
    MANAclust_results = in_stats[var][mom_row]
    MANAclust_results[MANAclust_results==np.nan]=0
    random_results[random_results==np.nan]=0
    paired_t_test_t, paired_t_test_p = ttest_rel(MANAclust_results, random_results)
    print(MANAclust_results)
    print(random_results)
    print(var,"paired t-test:",paired_t_test_t, paired_t_test_p)

    plt.clf()
    sns.violinplot(x="method", y=var,# hue="method",
                        scale = 'width', 
                        data=in_stats, palette="muted")
    out_file = out_dir + feature_type + '_feature_selection_'+var+".png"
    print('\n\n',var,var in ['specificity', 'accuracy'],'\n\n')
    # if var not in ['specificity', 'accuracy']:
    #     plt.ylim(0,1.05)
    # else:
    #     plt.ylim(0.8,1.05)
    ax = sns.swarmplot(x="method", y=var,# hue="method",
                  data=in_stats, palette="muted",
                  edgecolor = 'white', linewidth=1, s=22)
    ax.set_title("t="+str(paired_t_test_t)+"\np="+str(paired_t_test_p))
    plt.savefig(out_file,
            dpi=600,
            bbox_inches='tight')
    plt.clf()

    if var != 'specificity':
        do_mi_line_plot(in_stats, paired_t_test_t, paired_t_test_p, out_dir, 
                        y=var, name=feature_type, y_min = 0, y_max = 1.05)
    else:
        do_mi_line_plot(in_stats, paired_t_test_t, paired_t_test_p, out_dir, 
                        y=var, name=feature_type, y_min = 0.8, y_max = 1.05)
    
    plt.clf()


def get_feat_selection_stats(in_stats, name, out_dir):
    print('\n\t\tname:',name)
    print(in_stats)
    ## compare MANAclust
    #     "accuracy":[accuracy],
    # "recall":[recall],
    # "precision":[precision],
    # "specificity":[specificity],
    # "F1":[F_one]}
    variables = ["accuracy","recall","precision","specificity","F1"]
    for variable in variables:
        do_feat_select_plot(in_stats,
                            variable,
                            out_dir,
                            name)
    in_stats.to_csv(out_dir+name+'_feature_selection_stats.tsv',sep='\t')


def main():
    args = parse_args()

    seed = args.seed
    random.seed(seed)
    np.random.seed(seed)
    out_dir = process_dir(args.out_dir)
    results_dir = process_dir(out_dir+"/results")
    print('making the datasets')

    all_datasets, all_group_vectors = make_all_datasets(out_dir, 
                                                        args.iters, 
                                                        num_cat = args.num_cat_datasets, 
                                                        num_num = args.num_num_datasets, 
                                                        samples = args.samples,
                                                        num_cat_groups = args.num_cat_groups, 
                                                        num_cat_rand_feat = args.num_cat_rand_feat,
                                                        num_cat_real_feat = args.num_cat_real_feat,
                                                        cat_noise = args.cat_noise,
                                                        cat_percent_missing = args.cat_missing)
    all_feat_mi=[]
    all_full_mi=[]
    all_feat_purity = []
    all_full_purity = []
    cat_feat_select_stats = None
    num_feat_select_stats = None
    for i in range(len(all_datasets)):
        ## perform the clustering on the synthetic datset
        print('\n\n\nclustering ome #',i,"\n\n\n")
        temp_outdir = process_dir(out_dir + str(i))
        temp_results = process_dir(temp_outdir+'/results')
        temp_clustered_ome = cluster_omes(all_datasets[i]["cat"],all_datasets[i]["num"], out_dir = temp_outdir)
        temp_group_vector = get_group_from_clust_ome(temp_clustered_ome)
        ## write the results and plot some stats at the per-ome level
        write_ome_results(temp_results, temp_clustered_ome)
        plot_ome_results(temp_clustered_ome, temp_results, temp_group_vector)
        ## get clusteringa accuracy statistics 
        temp_feat_mi, temp_full_mi, feat_select_purity, full_purity = get_performance_stats(temp_group_vector, temp_clustered_ome, temp_results)
        all_feat_mi.append(temp_feat_mi)
        all_full_mi.append(temp_full_mi)
        all_feat_purity.append(feat_select_purity)
        all_full_purity.append(full_purity)
        ## check accuracy of feature selection
        temp_cat_df, temp_num_df = get_accuracy_of_feat_select(temp_clustered_ome, i)
        if type(cat_feat_select_stats) != pd.core.frame.DataFrame:
            cat_feat_select_stats = deepcopy(temp_cat_df)
            num_feat_select_stats = deepcopy(temp_num_df)
        else:
            cat_feat_select_stats = cat_feat_select_stats.append(temp_cat_df, ignore_index = True)
            num_feat_select_stats = num_feat_select_stats.append(temp_num_df, ignore_index = True)
    print(len(all_feat_mi))
    print(len(all_full_mi))
    print(len(all_feat_purity))
    print(len(all_full_purity))
    get_mi_stats(all_feat_mi, all_full_mi, all_feat_purity, all_full_purity, results_dir)

    ## get the stats for the feature selection process
    get_feat_selection_stats(cat_feat_select_stats, "categorical", results_dir)
    get_feat_selection_stats(num_feat_select_stats, "numeric", results_dir)


if __name__ == '__main__':
    main()


import os
import random
import numpy as np
import seaborn as sns
import argparse
from matplotlib import pyplot as plt

from mana_clust.mana_clust import cluster_omes
from mana_clust.common_functions import write_table, process_dir



def get_group_assignments(num_groups = 4, samples = 300, Gaussian=True):
	if Gaussian:
		p = np.ones(num_groups)/num_groups
	else:
		p = np.arange(num_groups)+1/num_groups
	group_selection_vector = np.arange(num_groups)
	all_group_annotations = []
	for i in range(samples):
		temp_group = np.random.choice(group_selection_vector, p=p)
		all_group_annotations.append(temp_group)
	return(all_group_annotations)


def get_col_names(all_features, real_features):
	## get the column names
	col_names = []
	for i in range(all_features):
		temp_name = "feature_"+str(i)+"_"
		if i < real_features:
			temp_name+="real"
		else:
			temp_name+="random"
		col_names.append(temp_name)
	col_names = np.array([col_names])
	return(col_names)


def get_cat_dataset(num_groups = 2, samples = 300, random_features = 150, percent_noise = .20, percent_missing = .25, real_features = 15):
	ome_array = np.chararray((samples, random_features+real_features), itemsize=2)
	all_features = real_features + random_features
	## first figure out which sample will go_with_which group
	temp_group_assignemnts = get_group_assignments(samples = samples, num_groups = num_groups)
	
	col_names = get_col_names(all_features, real_features)

	## generate the data
	real = list("ABCDEFGHIJKLMNOPQRSTUVWXY")
	for i in range(samples):
		for j in range(all_features):
			if j < real_features:
				## assign real features
				temp_group = temp_group_assignemnts[i]
				temp_val = real[temp_group]
				ome_array[i,j]=temp_val
			else:
				## assign the random features
				temp_val = random.choice(real[:num_groups])
				ome_array[i,j]=temp_val
	
	## introduce random noise
	num_noise = int(samples * all_features * percent_noise)
	rows = np.arange(ome_array.shape[0])
	cols = np.arange(ome_array.shape[1])
	for m in range(num_noise):
		## 
		temp_row = random.choice(rows)
		temp_col = random.choice(cols)
		ome_array[temp_row, temp_col]="Z"

	## introduce random missing values
	num_missing = int(samples * all_features * percent_missing)
	rows = np.arange(ome_array.shape[0])
	cols = np.arange(ome_array.shape[1])
	for m in range(num_missing):
		## 
		temp_row = random.choice(rows)
		temp_col = random.choice(cols)
		ome_array[temp_row, temp_col]="NA"

	## concatenate with title
	ome_array = np.concatenate((col_names, ome_array), axis = 0)

	return(temp_group_assignemnts, ome_array)


def get_num_dataset(samples = 300, num_groups = 3, random_features = 10000, real_features = 200, do_plotting = False):
	all_features = random_features+real_features
	ome_array = np.zeros((random_features+real_features, samples))
	master_points = np.zeros((random_features+real_features, num_groups))
	master_points[:real_features,:] += np.random.rand(real_features, num_groups)*1
	#print(master_points.shape)
	
	row_names = get_col_names(all_features, real_features)

	## first figure out which sample will go_with_which group
	temp_group_assignemnts = get_group_assignments(samples = samples, num_groups = num_groups)
	#print(temp_group_assignemnts)

	## go through each point, and generate it's simulated 'ome'
	for i in range(samples):
		temp_group = temp_group_assignemnts[i]
		ome_array[:,i] += master_points[:,temp_group] + np.random.rand(all_features,)

	if do_plotting:
		sns.clustermap(ome_array[:(real_features+real_features),:])
		plt.show()
		plt.clf()

		sns.clustermap(ome_array)
		plt.show()
		plt.clf()

	## concatenate with features_names
	ome_array = np.concatenate((np.transpose(row_names), np.array(ome_array,dtype=str)), axis = 1)

	return(temp_group_assignemnts, ome_array)


def get_all_final_groups(group_listings):
	group_listings = np.transpose(np.array(group_listings,dtype=str))
	final_group_names = ['variables']
	only_group = []
	for i in range(group_listings.shape[0]):
		temp_group = '_'.join(group_listings[i,:].tolist())
		final_group_names.append("subject_"+str(i)+"|"+temp_group)
		only_group.append(temp_group)
	return(np.array([final_group_names]), only_group)


def get_all_datasets(num_cat = 1, 
	                 num_num = 2, 
	                 samples= 1000, 
	                 num_cat_groups = 2, 
	                 num_cat_rand_feat = 100,
	                 num_cat_real_feat = 5,
	                 cat_noise = 0.20,
	                 cat_percent_missing = 0.25):
	cat_datasets = []
	cat_datasets_groups = []
	num_datasets = []
	num_datasets_groups = []
	for i in range(num_cat):
		temp_group, temp_cat = get_cat_dataset(samples = samples,
			                                   num_groups = num_cat_groups,
			                                   random_features = num_cat_rand_feat,
			                                   percent_noise = cat_noise,
			                                   percent_missing = cat_percent_missing,
			                                   real_features = num_cat_real_feat)
		cat_datasets.append(temp_cat)
		cat_datasets_groups.append(temp_group)
	for i in range(num_num):
		temp_group, temp_num = get_num_dataset(samples = samples)
		num_datasets.append(temp_num)
		num_datasets_groups.append(temp_group)
	

	final_groups, only_group = get_all_final_groups(cat_datasets_groups+num_datasets_groups)
	## concatenate final sample names to each ome
	for i in range(len(cat_datasets)):
		cat_datasets[i] = np.concatenate((np.transpose(final_groups),cat_datasets[i]), axis = 1)
	for i in range(len(num_datasets)):
		num_datasets[i] = np.concatenate((final_groups,num_datasets[i]), axis = 0)

	return(cat_datasets,num_datasets, only_group)


def write_datasets(num_cat = 1, 
	               num_num = 2, 
	               samples = 1000, 
	               num_cat_groups = 2, 
	               num_cat_rand_feat = 100,
	               num_cat_real_feat = 5,
	               cat_noise = 0.20,
	               cat_percent_missing = 0.25,
	               output_dir="/home/scott/bin/momcluster/lib/synthetic_data"):
	## process the output dir
	if output_dir[-1]!='/':
		output_dir+='/'
	if not os.path.isdir(output_dir):
		os.makedirs(output_dir)
	## write the data
	cat_data, num_data, group_vector = get_all_datasets(num_cat = num_cat, 
		                                                num_num = num_num, 
		                                                samples = samples,
		                                                num_cat_groups = num_cat_groups, 
										                num_cat_rand_feat = num_cat_rand_feat,
										                num_cat_real_feat = num_cat_real_feat,
										                cat_noise = cat_noise,
										                cat_percent_missing = cat_percent_missing)
	for cat in cat_data:
		print(cat)
	for num in num_data:
		print(num)
	cat_files = []
	for i in range(len(cat_data)):
		cat_files.append(output_dir+'categorical_data_file_'+str(i)+'.tsv')
		write_table(cat_data[i],cat_files[-1])
	num_files = []
	for i in range(len(num_data)):
		num_files.append(output_dir+'numeric_data_file_'+str(i)+'.tsv')
		write_table(num_data[i],num_files[-1])
	return(cat_files, num_files, group_vector)

def parse_args():
	parser = argparse.ArgumentParser()

	parser.add_argument(
		'-out_dir',
		type=str)

	parser.add_argument(
		'-iters','-iterations',
		type=int)

	parser.add_argument(
		'-seed',
		default = 123456789,
		type=int)

	args = parser.parse_args()
	return(args)

# def process_dir(in_dir):
# 	## process the output dir
# 	if in_dir[-1]!='/':
# 		in_dir+='/'
# 	if not os.path.isdir(in_dir):
# 		os.makedirs(in_dir)
# 	return(in_dir)

def main():
	args = parse_args()
	seed = args.seed
	random.seed(seed)
	np.random.seed(seed)
	all_datasets = make_all_datasets(args.out_dir, args.iters)

def make_all_datasets(out_dir, 
	                  iters, 
	                  num_cat = 1, 
	                  num_num = 2, 
	                  samples = 1000,
		              num_cat_groups = 2, 
		              num_cat_rand_feat = 100,
		              num_cat_real_feat = 5,
		              cat_noise = 0.20,
		              cat_percent_missing = 0.25):
	master_output_dir = process_dir(out_dir)
	all_datasets = []
	all_group_vectors = []
	for i in range(iters):
		next_out_dir = process_dir(master_output_dir+"/"+str(i))
		cat_files, num_files, group_vector=write_datasets(num_cat = num_cat, 
			                                              num_num = num_num, 
			                                              samples = samples, 
			                                              output_dir=next_out_dir,
			                                              num_cat_rand_feat = num_cat_rand_feat,
											              num_cat_real_feat = num_cat_real_feat,
											              cat_noise = cat_noise,
											              cat_percent_missing = cat_percent_missing)
		all_datasets.append({'cat':cat_files, 'num':num_files})
		all_group_vectors.append(group_vector)
	return(all_datasets, all_group_vectors)



if __name__ == '__main__':
	main()

